@@ -18,7 +18,7 @@
  * This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY
  * KIND, either express or implied.
  *
- * $Id: http.c,v 1.366 2008-03-27 13:07:12 bagder Exp $
+ * $Id: http.c,v 1.371 2008-05-19 20:57:28 bagder Exp $
  ***************************************************************************/
 
 #include "setup.h"
@@ -104,11 +104,16 @@
 /* The last #include file should be: */
 #include "memdebug.h"
 
+/* Default proxy timeout in milliseconds */
+#define PROXY_TIMEOUT (3600*1000)
 
 /*
  * Forward declarations.
  */
 
+static int http_getsock_do(struct connectdata *conn,
+                           curl_socket_t *socks,
+                           int numsocks);
 static CURLcode https_connecting(struct connectdata *conn, bool *done);
 #ifdef USE_SSL
 static int https_getsock(struct connectdata *conn,
@@ -129,7 +134,7 @@ const struct Curl_handler Curl_handler_http = {
   ZERO_NULL,                            /* connecting */
   ZERO_NULL,                            /* doing */
   ZERO_NULL,                            /* proto_getsock */
-  ZERO_NULL,                            /* doing_getsock */
+  http_getsock_do,                      /* doing_getsock */
   ZERO_NULL,                            /* disconnect */
   PORT_HTTP,                            /* defport */
   PROT_HTTP,                            /* protocol */
@@ -149,7 +154,7 @@ const struct Curl_handler Curl_handler_https = {
   https_connecting,                     /* connecting */
   ZERO_NULL,                            /* doing */
   https_getsock,                        /* proto_getsock */
-  ZERO_NULL,                            /* doing_getsock */
+  http_getsock_do,                      /* doing_getsock */
   ZERO_NULL,                            /* disconnect */
   PORT_HTTPS,                           /* defport */
   PROT_HTTP | PROT_HTTPS | PROT_SSL     /* protocol */
@@ -338,9 +343,13 @@ static CURLcode perhapsrewind(struct connectdata *conn)
      */
     conn->bits.close = TRUE;
     data->req.size = 0; /* don't download any more than 0 bytes */
+
+    /* There still is data left to send, but this connection is marked for
+       closure so we can safely do the rewind right now */
   }
 
   if(bytessent)
+    /* we rewind now at once since if we already sent something */
     return Curl_readrewind(conn);
 
   return CURLE_OK;
@@ -1221,7 +1230,7 @@ CURLcode Curl_proxyCONNECT(struct connectdata *conn,
   CURLcode result;
   int res;
   long timeout =
-    data->set.timeout?data->set.timeout:3600000; /* in milliseconds */
+    data->set.timeout?data->set.timeout:PROXY_TIMEOUT; /* in milliseconds */
   curl_socket_t tunnelsocket = conn->sock[sockindex];
   curl_off_t cl=0;
   bool closeConnection = FALSE;
@@ -1604,10 +1613,20 @@ CURLcode Curl_proxyCONNECT(struct connectdata *conn,
       if(error)
         return CURLE_RECV_ERROR;
 
-      if(data->info.httpproxycode != 200)
+      if(data->info.httpproxycode != 200) {
         /* Deal with the possibly already received authenticate
            headers. 'newurl' is set to a new URL if we must loop. */
-        Curl_http_auth_act(conn);
+        result = Curl_http_auth_act(conn);
+        if(result)
+          return result;
+
+        if(conn->bits.close)
+          /* the connection has been marked for closure, most likely in the
+             Curl_http_auth_act() function and thus we can kill it at once
+             below
+          */
+          closeConnection = TRUE;
+      }
 
       if(closeConnection && data->req.newurl) {
         /* Connection closed by server. Don't use it anymore */
@@ -1710,6 +1729,19 @@ CURLcode Curl_http_connect(struct connectdata *conn, bool *done)
   return CURLE_OK;
 }
 
+/* this returns the socket to wait for in the DO and DOING state for the multi
+   interface and then we're always _sending_ a request and thus we wait for
+   the single socket to become writable only */
+static int http_getsock_do(struct connectdata *conn,
+                           curl_socket_t *socks,
+                           int numsocks)
+{
+  /* write mode */
+  (void)numsocks; /* unused, we trust it to be at least 1 */
+  socks[0] = conn->sock[FIRSTSOCKET];
+  return GETSOCK_WRITESOCK(0);
+}
+
 static CURLcode https_connecting(struct connectdata *conn, bool *done)
 {
   CURLcode result;
@@ -2506,7 +2538,14 @@ CURLcode Curl_http(struct connectdata *conn, bool *done)
       return CURLE_HTTP_POST_ERROR;
     }
 
-    /* set the read function to read from the generated form data */
+    /* Get the currently set callback function pointer and store that in the
+       form struct since we might want the actual user-provided callback later
+       on. The conn->fread_func pointer itself will be changed for the
+       multipart case to the function that returns a multipart formatted
+       stream. */
+    http->form.fread_func = conn->fread_func;
+
+    /* Set the read function to read from the generated form data */
     conn->fread_func = (curl_read_callback)Curl_FormReader;
     conn->fread_in = &http->form;
 
