@@ -25,9 +25,6 @@
 #ifdef HAVE_SYS_PARAM_H
 #include <sys/param.h>
 #endif
-#ifdef HAVE_SYS_TYPES_H
-#include <sys/types.h>
-#endif
 #ifdef HAVE_SYS_SOCKET_H
 #include <sys/socket.h>
 #endif
@@ -55,13 +52,6 @@
 #define INADDRSZ         4
 #define INT16SZ          2
 
-#ifdef WIN32
-#define EAFNOSUPPORT    WSAEAFNOSUPPORT
-#define SET_ERRNO(e)    WSASetLastError(errno = (e))
-#else
-#define SET_ERRNO(e)    errno = e
-#endif
-
 /*
  * Format an IPv4 address, more or less like inet_ntoa().
  *
@@ -74,17 +64,22 @@ static char *inet_ntop4 (const unsigned char *src, char *dst, size_t size)
 {
 #if defined(HAVE_INET_NTOA_R_2_ARGS)
   const char *ptr;
-  curlassert(size >= 16);
+  DEBUGASSERT(size >= 16);
   ptr = inet_ntoa_r(*(struct in_addr*)src, dst);
   return (char *)memmove(dst, ptr, strlen(ptr)+1);
 
 #elif defined(HAVE_INET_NTOA_R)
+
+#if defined(HAVE_INT_INET_NTOA_R)
+  return inet_ntoa_r(*(struct in_addr*)src, dst, size)? NULL: dst;
+#else
   return inet_ntoa_r(*(struct in_addr*)src, dst, size);
+#endif
 
 #else
   const char *addr = inet_ntoa(*(struct in_addr*)src);
 
-  if (strlen(addr) >= size)
+  if(strlen(addr) >= size)
   {
     SET_ERRNO(ENOSPC);
     return (NULL);
@@ -130,23 +125,23 @@ static char *inet_ntop6 (const unsigned char *src, char *dst, size_t size)
 
   for (i = 0; i < (IN6ADDRSZ / INT16SZ); i++)
   {
-    if (words[i] == 0)
+    if(words[i] == 0)
     {
-      if (cur.base == -1)
+      if(cur.base == -1)
         cur.base = i, cur.len = 1;
       else
         cur.len++;
     }
-    else if (cur.base != -1)
+    else if(cur.base != -1)
     {
-      if (best.base == -1 || cur.len > best.len)
+      if(best.base == -1 || cur.len > best.len)
          best = cur;
       cur.base = -1;
     }
   }
-  if ((cur.base != -1) && (best.base == -1 || cur.len > best.len))
+  if((cur.base != -1) && (best.base == -1 || cur.len > best.len))
      best = cur;
-  if (best.base != -1 && best.len < 2)
+  if(best.base != -1 && best.len < 2)
      best.base = -1;
 
   /* Format the result.
@@ -156,24 +151,24 @@ static char *inet_ntop6 (const unsigned char *src, char *dst, size_t size)
   {
     /* Are we inside the best run of 0x00's?
      */
-    if (best.base != -1 && i >= best.base && i < (best.base + best.len))
+    if(best.base != -1 && i >= best.base && i < (best.base + best.len))
     {
-      if (i == best.base)
+      if(i == best.base)
          *tp++ = ':';
       continue;
     }
 
     /* Are we following an initial run of 0x00s or any real hex?
      */
-    if (i != 0)
+    if(i != 0)
        *tp++ = ':';
 
     /* Is this address an encapsulated IPv4?
      */
-    if (i == 6 && best.base == 0 &&
+    if(i == 6 && best.base == 0 &&
         (best.len == 6 || (best.len == 5 && words[5] == 0xffff)))
     {
-      if (!inet_ntop4(src+12, tp, sizeof(tmp) - (tp - tmp)))
+      if(!inet_ntop4(src+12, tp, sizeof(tmp) - (tp - tmp)))
       {
         SET_ERRNO(ENOSPC);
         return (NULL);
@@ -186,13 +181,13 @@ static char *inet_ntop6 (const unsigned char *src, char *dst, size_t size)
 
   /* Was it a trailing run of 0x00's?
    */
-  if (best.base != -1 && (best.base + best.len) == (IN6ADDRSZ / INT16SZ))
+  if(best.base != -1 && (best.base + best.len) == (IN6ADDRSZ / INT16SZ))
      *tp++ = ':';
   *tp++ = '\0';
 
   /* Check for overflow, copy, and we're done.
    */
-  if ((size_t)(tp - tmp) > size)
+  if((size_t)(tp - tmp) > size)
   {
     SET_ERRNO(ENOSPC);
     return (NULL);
@@ -204,8 +199,14 @@ static char *inet_ntop6 (const unsigned char *src, char *dst, size_t size)
 /*
  * Convert a network format address to presentation format.
  *
- * Returns pointer to presentation format address (`buf'),
- * Returns NULL on error (see errno).
+ * Returns pointer to presentation format address (`buf').
+ * Returns NULL on error and errno set with the specific
+ * error, EAFNOSUPPORT or ENOSPC.
+ *
+ * On Windows we store the error in the thread errno, not
+ * in the winsock error code. This is to avoid loosing the
+ * actual last winsock error. So use macro ERRNO to fetch the
+ * errno this funtion sets when returning NULL, not SOCKERRNO.
  */
 char *Curl_inet_ntop(int af, const void *src, char *buf, size_t size)
 {
