@@ -20,7 +20,7 @@
  * This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY
  * KIND, either express or implied.
  *
- * $Id: setup.h,v 1.136 2007-11-08 18:13:55 yangtse Exp $
+ * $Id: setup.h,v 1.137 2008-04-22 22:53:54 danf Exp $
  ***************************************************************************/
 
 #ifdef HTTP_ONLY
@@ -36,7 +36,7 @@
  * Define WIN32 when build target is Win32 API
  */
 
-#if (defined(_WIN32) || defined(__WIN32__)) && !defined(WIN32)
+#if (defined(_WIN32) || defined(__WIN32__)) && !defined(WIN32) && !defined(__SYMBIAN32__)
 #define WIN32
 #endif
 
@@ -65,6 +65,10 @@
 #include "amigaos.h"
 #endif
 
+#ifdef __SYMBIAN32__
+#include "config-symbian.h"
+#endif
+
 #ifdef __OS400__
 #include "config-os400.h"
 #include "setup-os400.h"
@@ -268,6 +272,10 @@ extern char * strtok_r(char *s, const char *delim, char **last);
 extern struct tm * gmtime_r(const time_t * const timep, struct tm *tmp);
 #endif
 
+#ifdef __SYMBIAN32__
+#undef HAVE_ALARM
+#endif
+
 #define DIR_CHAR      "/"
 #ifndef DOT_CHAR
 #define DOT_CHAR      "."
