@@ -20,7 +20,7 @@
  * This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY
  * KIND, either express or implied.
  *
- * $Id: setup_once.h,v 1.29 2007-10-24 14:39:07 yangtse Exp $
+ * $Id: setup_once.h,v 1.30 2008-05-21 14:04:14 yangtse Exp $
  ***************************************************************************/
 
 
@@ -107,11 +107,10 @@ struct timeval {
  * definition is present the other one also is available.
  */
 
-#if defined(WIN32) && !defined(HAVE_SOCKLEN_T)
+#if defined(WIN32) && !defined(HAVE_CONFIG_H)
 #  if ( defined(_MSC_VER) && !defined(INET_ADDRSTRLEN) ) || \
       (!defined(_MSC_VER) && !defined(HAVE_WS2TCPIP_H) )
 #    define socklen_t int
-#    define HAVE_SOCKLEN_T
 #  endif
 #endif
 
