@@ -1,6 +1,6 @@
 #!/bin/sh
 
-# $Id: initscript.sh,v 1.3 2008-01-16 16:04:47 patrickm Exp $
+# $Id: initscript.sh,v 1.5 2008-05-20 10:21:50 patrickm Exp $
 
 case "${SCRIPTDIR}" in
 /*)     ;;
@@ -40,10 +40,10 @@ SRVPGM="CURL.${SONAME}"         # Service program.
 TGTCCSID='500'                  # Target CCSID of objects
 DEBUG='*ALL'                    # Debug level
 OPTIMIZE='10'                   # Optimisation level
-OUTPUT='*NONE'                   # Compilation output option.
+OUTPUT='*NONE'                  # Compilation output option.
 TGTRLS='V5R2M0'                 # Target OS release
 
-export TARGETLIB STATBNDDIR DYNBNDDIR SRVPGM TGTCCSID DEBUG OPTIMIZE OUTPUTC
+export TARGETLIB STATBNDDIR DYNBNDDIR SRVPGM TGTCCSID DEBUG OPTIMIZE OUTPUT
 export TGTRLS
 
 
@@ -156,7 +156,7 @@ db2_name()
 
 {
         basename "${1}"                                                 |
-        tr '[a-z]' '[A-Z]'                                              |
+        tr '[a-z-]' '[A-Z_]'                                              |
         sed -e 's/\..*//'                                               \
             -e 's/^\(..........\).*/\1/'
 }
