@@ -5,7 +5,7 @@
  *                            | (__| |_| |  _ <| |___
  *                             \___|\___/|_| \_\_____|
  *
- * Copyright (C) 1998 - 2006, Daniel Stenberg, <daniel@haxx.se>, et al.
+ * Copyright (C) 1998 - 2008, Daniel Stenberg, <daniel@haxx.se>, et al.
  *
  * This software is licensed as described in the file COPYING, which
  * you should have received as part of this distribution. The terms
@@ -18,7 +18,7 @@
  * This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY
  * KIND, either express or implied.
  *
- * $Id: writeout.c,v 1.29 2006-02-23 21:33:02 bagder Exp $
+ * $Id: writeout.c,v 1.32 2008-04-30 21:20:09 bagder Exp $
  ***************************************************************************/
 
 #include "setup.h"
@@ -60,6 +60,8 @@ typedef enum {
   VAR_NUM_CONNECTS,
   VAR_REDIRECT_TIME,
   VAR_REDIRECT_COUNT,
+  VAR_FTP_ENTRY_PATH,
+  VAR_REDIRECT_URL,
   VAR_NUM_OF_VARS /* must be the last */
 } replaceid;
 
@@ -72,6 +74,7 @@ struct variable {
 static const struct variable replacements[]={
   {"url_effective", VAR_EFFECTIVE_URL},
   {"http_code", VAR_HTTP_CODE},
+  {"response_code", VAR_HTTP_CODE},
   {"http_connect", VAR_HTTP_CODE_PROXY},
   {"time_total", VAR_TOTAL_TIME},
   {"time_namelookup", VAR_NAMELOOKUP_TIME},
@@ -88,13 +91,15 @@ static const struct variable replacements[]={
   {"num_connects", VAR_NUM_CONNECTS},
   {"time_redirect", VAR_REDIRECT_TIME},
   {"num_redirects", VAR_REDIRECT_COUNT},
+  {"ftp_entry_path", VAR_FTP_ENTRY_PATH},
+  {"redirect_url", VAR_REDIRECT_URL},
   {NULL, VAR_NONE}
 };
 
-void ourWriteOut(CURL *curl, char *writeinfo)
+void ourWriteOut(CURL *curl, const char *writeinfo)
 {
   FILE *stream = stdout;
-  char *ptr=writeinfo;
+  const char *ptr=writeinfo;
   char *stringp;
   long longinfo;
   double doubleinfo;
@@ -213,6 +218,19 @@ void ourWriteOut(CURL *curl, char *writeinfo)
                     curl_easy_getinfo(curl, CURLINFO_CONTENT_TYPE, &stringp))
                    && stringp)
                   fputs(stringp, stream);
+                break;
+              case VAR_FTP_ENTRY_PATH:
+                if((CURLE_OK ==
+                    curl_easy_getinfo(curl, CURLINFO_FTP_ENTRY_PATH, &stringp))
+                   && stringp)
+                  fputs(stringp, stream);
+                break;
+              case VAR_REDIRECT_URL:
+                if((CURLE_OK ==
+                    curl_easy_getinfo(curl, CURLINFO_REDIRECT_URL, &stringp))
+                   && stringp)
+                  fputs(stringp, stream);
+                break;
               default:
                 break;
               }
