@@ -5,7 +5,7 @@
  *                            | (__| |_| |  _ <| |___
  *                             \___|\___/|_| \_\_____|
  *
- * $Id: lib501.c,v 1.5 2006-10-25 09:20:44 yangtse Exp $
+ * $Id: lib501.c,v 1.6 2008-05-22 21:49:52 danf Exp $
  */
 
 #include "test.h"
@@ -28,7 +28,7 @@ int test(char *URL)
     return TEST_ERR_MAJOR_BAD;
   }
 
-  curl_easy_setopt(curl, CURLOPT_HEADER, TRUE);
+  curl_easy_setopt(curl, CURLOPT_HEADER, 1L);
 
   res = curl_easy_perform(curl);
 
