@@ -5,7 +5,7 @@
  *                            | (__| |_| |  _ <| |___
  *                             \___|\___/|_| \_\_____|
  *
- * $Id: lib540.c,v 1.5 2007-10-02 16:05:28 yangtse Exp $
+ * $Id: lib540.c,v 1.6 2008-05-22 21:49:53 danf Exp $
  *
  * This is the 'proxyauth.c' test app posted by Shmulik Regev on the libcurl
  * mailing list on 10 Jul 2007, converted to a test case.
@@ -30,9 +30,9 @@ static void init(CURLM *cm, const char* url, const char* userpwd,
   curl_easy_setopt(eh, CURLOPT_URL, url);
   curl_easy_setopt(eh, CURLOPT_PROXY, PROXY);
   curl_easy_setopt(eh, CURLOPT_PROXYUSERPWD, userpwd);
-  curl_easy_setopt(eh, CURLOPT_PROXYAUTH, CURLAUTH_ANY);
-  curl_easy_setopt(eh, CURLOPT_VERBOSE, 1);
-  curl_easy_setopt(eh, CURLOPT_HEADER, 1);
+  curl_easy_setopt(eh, CURLOPT_PROXYAUTH, (long)CURLAUTH_ANY);
+  curl_easy_setopt(eh, CURLOPT_VERBOSE, 1L);
+  curl_easy_setopt(eh, CURLOPT_HEADER, 1L);
   curl_easy_setopt(eh, CURLOPT_HTTPHEADER, headers); /* custom Host: */
 
   curl_multi_add_handle(cm, eh);
