@@ -5,7 +5,7 @@
  *                            | (__| |_| |  _ <| |___
  *                             \___|\___/|_| \_\_____|
  *
- * $Id: lib541.c,v 1.2 2007-10-02 16:05:28 yangtse Exp $
+ * $Id: lib541.c,v 1.3 2008-05-22 21:49:53 danf Exp $
  */
 
 #include "setup.h" /* struct_stat etc. */
@@ -89,10 +89,10 @@ int test(char *URL)
   }
 
   /* enable uploading */
-  curl_easy_setopt(curl, CURLOPT_UPLOAD, TRUE) ;
+  curl_easy_setopt(curl, CURLOPT_UPLOAD, 1L);
 
   /* enable verbose */
-  curl_easy_setopt(curl, CURLOPT_VERBOSE, TRUE) ;
+  curl_easy_setopt(curl, CURLOPT_VERBOSE, 1L);
 
   /* specify target */
   curl_easy_setopt(curl,CURLOPT_URL, URL);
