@@ -19,7 +19,7 @@
 # This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY
 # KIND, either express or implied.
 #
-# $Id: runtests.pl,v 1.284 2008-01-18 09:18:59 yangtse Exp $
+# $Id: runtests.pl,v 1.295 2008-05-01 10:51:16 bagder Exp $
 ###########################################################################
 
 # Experimental hooks are available to run tests remotely on machines that
@@ -69,14 +69,21 @@ use Cwd;
 use sshhelp qw(
     $sshdexe
     $sshexe
+    $sftpexe
     $sshconfig
+    $sftpconfig
     $sshlog
+    $sftplog
+    $sftpcmds
     display_sshdconfig
     display_sshconfig
+    display_sftpconfig
     display_sshdlog
     display_sshlog
+    display_sftplog
     find_sshd
     find_ssh
+    find_sftp
     sshversioninfo
     );
 
@@ -113,6 +120,7 @@ my $SERVERIN="$LOGDIR/server.input"; # what curl sent the server
 my $SERVER2IN="$LOGDIR/server2.input"; # what curl sent the second server
 my $CURLLOG="$LOGDIR/curl.log"; # all command lines run
 my $FTPDCMD="$LOGDIR/ftpserver.cmd"; # copy ftp server instructions here
+my $SERVERLOGS_LOCK="$LOGDIR/serverlogs.lock"; # server logs advisor read lock
 
 # Normally, all test cases should be run, but at times it is handy to
 # simply run a particular one:
@@ -195,6 +203,9 @@ my $sshdvernum;  # for socks server, ssh daemon version number
 my $sshdverstr;  # for socks server, ssh daemon version string
 my $sshderror;   # for socks server, ssh daemon version error
 
+my $defserverlogslocktimeout = 20; # timeout to await server logs lock removal
+my $defpostcommanddelay = 0; # delay between command and postcheck sections
+
 #######################################################################
 # variables the command line options may set
 #
@@ -658,7 +669,9 @@ sub verifyftp {
 }
 
 #######################################################################
-# STUB for verifying scp/sftp
+# Verify that the ssh server has written out its pidfile, recovering
+# the pid from the file and returning it if a process with that pid is
+# actually alive.
 
 sub verifyssh {
     my ($proto, $ip, $port) = @_;
@@ -680,6 +693,43 @@ sub verifyssh {
     return $pid;
 }
 
+#######################################################################
+# Verify that we can connect to the sftp server, properly authenticate
+# with generated config and key files and run a simple remote pwd.
+
+sub verifysftp {
+    my ($proto, $ip, $port) = @_;
+    my $verified = 0;
+    # Find out sftp client canonical file name
+    my $sftp = find_sftp();
+    if(!$sftp) {
+        logmsg "RUN: SFTP server cannot find $sftpexe\n";
+        return -1;
+    }
+    # Find out ssh client canonical file name
+    my $ssh = find_ssh();
+    if(!$ssh) {
+        logmsg "RUN: SFTP server cannot find $sshexe\n";
+        return -1;
+    }
+    # Connect to sftp server, authenticate and run a remote pwd
+    # command using our generated configuration and key files
+    my $cmd = "$sftp -b $sftpcmds -F $sftpconfig -S $ssh $ip > $sftplog 2>&1";
+    my $res = runclient($cmd);
+    # Search for pwd command response in log file
+    if(open(SFTPLOGFILE, "<$sftplog")) {
+        while(<SFTPLOGFILE>) {
+            if(/^Remote working directory: /) {
+                $verified = 1;
+                last;
+            }
+        }
+        close(SFTPLOGFILE);
+    }
+    return $verified;
+}
+
+
 #######################################################################
 # STUB for verifying socks
 
@@ -749,7 +799,6 @@ sub verifyserver {
 sub runhttpserver {
     my ($verbose, $ipv6) = @_;
     my $RUNNING;
-    my $pid;
     my $pidfile = $HTTPPIDFILE;
     my $port = $HTTPPORT;
     my $ip = $HOSTIP;
@@ -769,11 +818,11 @@ sub runhttpserver {
         return (0,0);
     }
 
-    $pid = checkserver($pidfile);
-
+    my $pid = checkserver($pidfile);
     if($pid > 0) {
         stopserver($pid);
     }
+    unlink($pidfile);
 
     my $flag=$debugprotocol?"-v ":"";
     my $dir=$ENV{'srcdir'};
@@ -788,18 +837,21 @@ sub runhttpserver {
     if($httppid <= 0 || !kill(0, $httppid)) {
         # it is NOT alive
         logmsg "RUN: failed to start the HTTP$nameext server\n";
+        stopserver("$pid2");
         $doesntrun{$pidfile} = 1;
         return (0,0);
     }
 
     # Server is up. Verify that we can speak to it.
-    if(!verifyserver("http", $ip, $port)) {
+    my $pid3 = verifyserver("http", $ip, $port);
+    if(!$pid3) {
         logmsg "RUN: HTTP$nameext server failed verification\n";
         # failed to talk to it properly. Kill the server and return failure
         stopserver("$httppid $pid2");
         $doesntrun{$pidfile} = 1;
         return (0,0);
     }
+    $pid2 = $pid3;
 
     if($verbose) {
         logmsg "RUN: HTTP$nameext server is now running PID $httppid\n";
@@ -834,12 +886,12 @@ sub runhttpsserver {
         return (0,0);
     }
 
-    my $pid=checkserver($pidfile);
-
+    my $pid = checkserver($pidfile);
     if($pid > 0) {
         # kill previous stunnel!
         stopserver($pid);
     }
+    unlink($pidfile);
 
     my $flag=$debugprotocol?"-v ":"";
     my $cmd="$perl $srcdir/httpsserver.pl $flag -p https -s \"$stunnel\" -d $srcdir -r $HTTPPORT $HTTPSPORT";
@@ -855,13 +907,15 @@ sub runhttpsserver {
     }
 
     # Server is up. Verify that we can speak to it.
-    if(!verifyserver("https", $ip, $HTTPSPORT)) {
+    my $pid3 = verifyserver("https", $ip, $HTTPSPORT);
+    if(!$pid3) {
         logmsg "RUN: HTTPS server failed verification\n";
         # failed to talk to it properly. Kill the server and return failure
         stopserver("$httpspid $pid2");
         $doesntrun{$pidfile} = 1;
         return (0,0);
     }
+    # Here pid3 is actually the pid returned by the unsecure-http server.
 
     if($verbose) {
         logmsg "RUN: HTTPS server is now running PID $httpspid\n";
@@ -900,9 +954,10 @@ sub runftpserver {
     }
 
     my $pid = checkserver($pidfile);
-    if($pid >= 0) {
+    if($pid > 0) {
         stopserver($pid);
     }
+    unlink($pidfile);
 
     # start our server:
     my $flag=$debugprotocol?"-v ":"";
@@ -917,27 +972,28 @@ sub runftpserver {
     } else {
         $addr = $HOSTIP;
     }
+
     $cmd="$perl $srcdir/ftpserver.pl --pidfile $pidfile $flag --port $port --addr \"$addr\"";
-
-    unlink($pidfile);
-
     my ($ftppid, $pid2) = startnew($cmd, $pidfile, 15, 0);
 
     if($ftppid <= 0 || !kill(0, $ftppid)) {
         # it is NOT alive
         logmsg "RUN: failed to start the FTP$id$nameext server\n";
+        stopserver("$pid2");
         $doesntrun{$pidfile} = 1;
         return (0,0);
     }
 
     # Server is up. Verify that we can speak to it.
-    if(!verifyserver("ftp", $ip, $port)) {
+    my $pid3 = verifyserver("ftp", $ip, $port);
+    if(!$pid3) {
         logmsg "RUN: FTP$id$nameext server failed verification\n";
         # failed to talk to it properly. Kill the server and return failure
         stopserver("$ftppid $pid2");
         $doesntrun{$pidfile} = 1;
         return (0,0);
     }
+    $pid2 = $pid3;
 
     if($verbose) {
         logmsg "RUN: FTP$id$nameext server is now running PID $ftppid\n";
@@ -972,12 +1028,12 @@ sub runftpsserver {
         return (0,0);
     }
 
-    my $pid=checkserver($pidfile);
-
+    my $pid = checkserver($pidfile);
     if($pid > 0) {
         # kill previous stunnel!
         stopserver($pid);
     }
+    unlink($pidfile);
 
     my $flag=$debugprotocol?"-v ":"";
     my $cmd="$perl $srcdir/httpsserver.pl $flag -p ftps -s \"$stunnel\" -d $srcdir -r $FTPPORT $FTPSPORT";
@@ -993,13 +1049,15 @@ sub runftpsserver {
     }
 
     # Server is up. Verify that we can speak to it.
-    if(!verifyserver("ftps", $ip, $FTPSPORT)) {
+    my $pid3 = verifyserver("ftps", $ip, $FTPSPORT);
+    if(!$pid3) {
         logmsg "RUN: FTPS server failed verification\n";
         # failed to talk to it properly. Kill the server and return failure
         stopserver("$ftpspid $pid2");
         $doesntrun{$pidfile} = 1;
         return (0,0);
     }
+    # Here pid3 is actually the pid returned by the unsecure-ftp server.
 
     if($verbose) {
         logmsg "RUN: FTPS server is now running PID $ftpspid\n";
@@ -1038,9 +1096,10 @@ sub runtftpserver {
     }
 
     my $pid = checkserver($pidfile);
-    if($pid >= 0) {
+    if($pid > 0) {
         stopserver($pid);
     }
+    unlink($pidfile);
 
     # start our server:
     my $flag=$debugprotocol?"-v ":"";
@@ -1051,27 +1110,28 @@ sub runtftpserver {
     if($ipv6) {
         $flag .="--ipv6 ";
     }
+
     $cmd="./server/tftpd --pidfile $pidfile $flag $port";
-
-    unlink($pidfile);
-
     my ($tftppid, $pid2) = startnew($cmd, $pidfile, 15, 0);
 
     if($tftppid <= 0 || !kill(0, $tftppid)) {
         # it is NOT alive
         logmsg "RUN: failed to start the TFTP$id$nameext server\n";
+        stopserver("$pid2");
         $doesntrun{$pidfile} = 1;
         return (0,0);
     }
 
     # Server is up. Verify that we can speak to it.
-    if(!verifyserver("tftp", $ip, $port)) {
+    my $pid3 = verifyserver("tftp", $ip, $port);
+    if(!$pid3) {
         logmsg "RUN: TFTP$id$nameext server failed verification\n";
         # failed to talk to it properly. Kill the server and return failure
         stopserver("$tftppid $pid2");
         $doesntrun{$pidfile} = 1;
         return (0,0);
     }
+    $pid2 = $pid3;
 
     if($verbose) {
         logmsg "RUN: TFTP$id$nameext server is now running PID $tftppid\n";
@@ -1122,19 +1182,35 @@ sub runsshserver {
         return (0,0);
     }
 
-    # server verification allows some extra time for the server to start up
-    # and gives us the opportunity of recovering the pid from the pidfile,
-    # which will be assigned to pid2 ONLY if pid2 was not already positive.
+    # ssh server verification allows some extra time for the server to start up
+    # and gives us the opportunity of recovering the pid from the pidfile, when
+    # this verification succeeds the recovered pid is assigned to pid2.
 
     my $pid3 = verifyserver("ssh",$ip,$port);
     if(!$pid3) {
         logmsg "RUN: SSH server failed verification\n";
+        # failed to fetch server pid. Kill the server and return failure
+        stopserver("$sshpid $pid2");
+        $doesntrun{$pidfile} = 1;
+        return (0,0);
+    }
+    $pid2 = $pid3;
+
+    # once it is known that the ssh server is alive, sftp server verification
+    # is performed actually connecting to it, authenticating and performing a
+    # very simple remote command.  This verification is tried only one time.
+
+    if(verifysftp("sftp",$ip,$port) < 1) {
+        logmsg "RUN: SFTP server failed verification\n";
         # failed to talk to it properly. Kill the server and return failure
+        display_sftplog();
+        display_sftpconfig();
+        display_sshdlog();
+        display_sshdconfig();
         stopserver("$sshpid $pid2");
         $doesntrun{$pidfile} = 1;
         return (0,0);
     }
-    $pid2 = $pid3 if($pid2 <= 0);
 
     if($verbose) {
         logmsg "RUN: SSH server is now running PID $pid2\n";
@@ -1256,7 +1332,7 @@ sub runsocksserver {
         $doesntrun{$pidfile} = 1;
         return (0,0);
     }
-    $pid2 = $pid3 if($pid2 <= 0);
+    $pid2 = $pid3;
 
     if($verbose) {
         logmsg "RUN: SOCKS server is now running PID $pid2\n";
@@ -1742,10 +1818,6 @@ sub singletest {
         last;
     }
 
-    if(!$why) {
-        $why = serverfortest($testnum);
-    }
-
     if(!$why) {
         my @keywords = getpart("info", "keywords");
         my $k;
@@ -1757,6 +1829,10 @@ sub singletest {
         }
     }
 
+    if(!$why) {
+        $why = serverfortest($testnum);
+    }
+
     if(!$why) {
         my @precheck = getpart("client", "precheck");
         $cmd = $precheck[0];
@@ -1885,13 +1961,11 @@ sub singletest {
         unlink($memdump);
     }
 
+    # create a (possibly-empty) file before starting the test
     my @inputfile=getpart("client", "file");
-    if(@inputfile) {
-        # we need to generate a file before this test is invoked
-        my %fileattr = getpartattr("client", "file");
-
-        my $filename=$fileattr{'name'};
-
+    my %fileattr = getpartattr("client", "file");
+    my $filename=$fileattr{'name'};
+    if(@inputfile || $filename) {
         if(!$filename) {
             logmsg "ERROR: section client=>file has no name attribute\n";
             return -1;
@@ -1916,6 +1990,22 @@ sub singletest {
         }
     }
 
+    my $serverlogslocktimeout = $defserverlogslocktimeout;
+    if($cmdhash{'timeout'}) {
+        # test is allowed to override default server logs lock timeout
+        if($cmdhash{'timeout'} =~ /(\d+)/) {
+            $serverlogslocktimeout = $1 if($1 >= 0);
+        }
+    }
+
+    my $postcommanddelay = $defpostcommanddelay;
+    if($cmdhash{'delay'}) {
+        # test is allowed to specify a delay after command is executed
+        if($cmdhash{'delay'} =~ /(\d+)/) {
+            $postcommanddelay = $1 if($1 > 0);
+        }
+    }
+
     my $cmdargs;
     if(!$tool) {
         # run curl, add -v for debug information output
@@ -2026,18 +2116,44 @@ sub singletest {
         }
     }
 
+    # If a server logs advisor read lock file exists, it is an indication
+    # that the server has not yet finished writing out all its log files,
+    # including server request log files used for protocol verification.
+    # So, if the lock file exists the script waits here a certain amount
+    # of time until the server removes it, or the given time expires.
+
+    if($serverlogslocktimeout) {
+        my $lockretry = $serverlogslocktimeout * 4;
+        while((-f $SERVERLOGS_LOCK) && $lockretry--) {
+            select(undef, undef, undef, 0.25);
+        }
+        if(($lockretry < 0) &&
+           ($serverlogslocktimeout >= $defserverlogslocktimeout)) {
+            logmsg "Warning: server logs lock timeout ",
+                   "($serverlogslocktimeout seconds) expired\n";
+        }
+    }
+
+    # Test harness ssh server does not have this synchronization mechanism,
+    # this implies that some ssh server based tests might need a small delay
+    # once that the client command has run to avoid false test failures.
+
+    sleep($postcommanddelay) if($postcommanddelay);
+
     # run the postcheck command
     my @postcheck= getpart("client", "postcheck");
     $cmd = $postcheck[0];
     chomp $cmd;
     subVariables \$cmd;
     if($cmd) {
+	logmsg "postcheck $cmd\n" if($verbose);
 	my $rc = runclient("$cmd");
-	if($rc != 0) {
+	# Must run the postcheck command in torture mode in order
+	# to clean up, but the result can't be relied upon.
+	if($rc != 0 && !$torture) {
 	    logmsg " postcheck FAILED\n";
 	    return 1;
 	}
-	logmsg "postchecked $cmd\n" if($verbose);
     }
 
     # remove the special FTP command file after each test!
@@ -2064,6 +2180,9 @@ sub singletest {
         # verify redirected stdout
         my @actual = loadarray($STDOUT);
 
+        # variable-replace in the stdout we have from the test case file
+        @validstdout = fixarray(@validstdout);
+
         # get all attributes
         my %hash = getpartattr("verify", "stdout");
 
@@ -2126,22 +2245,8 @@ sub singletest {
     }
 
     if(@protocol) {
-        my @out;
-        my $retry = 5;
-
-        # Verify the sent request. Sometimes, like in test 513 on some hosts,
-        # curl will return back faster than the server writes down the request
-        # to its file, so we might need to wait here for a while to see if the
-        # file gets written a bit later.
-
-        while($retry--) {
-            @out = loadarray($SERVERIN);
-
-            if(!$out[0]) {
-                # nothing there yet, wait a while and try again
-                sleep(1);
-            }
-        }
+        # Verify the sent request
+        my @out = loadarray($SERVERIN);
 
         # what to cut off from the live protocol sent by curl
         my @strip = getpart("verify", "strip");
@@ -2246,7 +2351,7 @@ sub singletest {
     }
     else {
         if(!$short) {
-            printf "\ncurl returned $cmdres, %s was expected\n", $errorcode;
+            printf "\ncurl returned $cmdres, %d was expected\n", $errorcode;
         }
         logmsg " exit FAILED\n";
         return 1;
@@ -3048,6 +3153,7 @@ if($skipped) {
         logmsg ")\n";
     }
 }
+
 if($total && ($ok != $total)) {
     exit 1;
 }
