@@ -5,7 +5,7 @@
  *                            | (__| |_| |  _ <| |___
  *                             \___|\___/|_| \_\_____|
  *
- * Copyright (C) 1998 - 2007, Daniel Stenberg, <daniel@haxx.se>, et al.
+ * Copyright (C) 1998 - 2008, Daniel Stenberg, <daniel@haxx.se>, et al.
  *
  * This software is licensed as described in the file COPYING, which
  * you should have received as part of this distribution. The terms
@@ -18,7 +18,7 @@
  * This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY
  * KIND, either express or implied.
  *
- * $Id: sockfilt.c,v 1.38 2007-10-09 23:21:29 yangtse Exp $
+ * $Id: sockfilt.c,v 1.51 2008-03-05 18:27:31 yangtse Exp $
  ***************************************************************************/
 
 /* Purpose
@@ -46,18 +46,45 @@
  *
  * (Source originally based on sws.c)
  */
+
+/*
+ * Signal handling notes for sockfilt
+ * ----------------------------------
+ *
+ * This program is a single-threaded process.
+ *
+ * This program is intended to be highly portable and as such it must be kept as
+ * simple as possible, due to this the only signal handling mechanisms used will
+ * be those of ANSI C, and used only in the most basic form which is good enough
+ * for the purpose of this program.
+ *
+ * For the above reason and the specific needs of this program signals SIGHUP,
+ * SIGPIPE and SIGALRM will be simply ignored on systems where this can be done.
+ * If possible, signals SIGINT and SIGTERM will be handled by this program as an
+ * indication to cleanup and finish execution as soon as possible.  This will be
+ * achieved with a single signal handler 'exit_signal_handler' for both signals.
+ *
+ * The 'exit_signal_handler' upon the first SIGINT or SIGTERM received signal
+ * will just set to one the global var 'got_exit_signal' storing in global var
+ * 'exit_signal' the signal that triggered this change.
+ *
+ * Nothing fancy that could introduce problems is used, the program at certain
+ * points in its normal flow checks if var 'got_exit_signal' is set and in case
+ * this is true it just makes its way out of loops and functions in structured
+ * and well behaved manner to achieve proper program cleanup and termination.
+ *
+ * Even with the above mechanism implemented it is worthwile to note that other
+ * signals might still be received, or that there might be systems on which it
+ * is not possible to trap and ignore some of the above signals.  This implies
+ * that for increased portability and reliability the program must be coded as
+ * if no signal was being ignored or handled at all.  Enjoy it!
+ */
+
 #include "setup.h" /* portability help from the lib directory */
 
-#include <stdio.h>
-#include <stdlib.h>
-#include <string.h>
-#include <stdarg.h>
+#ifdef HAVE_SIGNAL_H
 #include <signal.h>
-#include <time.h>
-#include <ctype.h>
-#include <sys/time.h>
-#include <sys/types.h>
-
+#endif
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #endif
@@ -92,11 +119,238 @@
 #define DEFAULT_LOGFILE "log/sockfilt.log"
 #endif
 
-#ifdef SIGPIPE
-static volatile int sigpipe;  /* Why? It's not used */
+const char *serverlogfile = (char *)DEFAULT_LOGFILE;
+
+bool verbose = FALSE;
+bool use_ipv6 = FALSE;
+unsigned short port = DEFAULT_PORT;
+unsigned short connectport = 0; /* if non-zero, we activate this mode */
+
+enum sockmode {
+  PASSIVE_LISTEN,    /* as a server waiting for connections */
+  PASSIVE_CONNECT,   /* as a server, connected to a client */
+  ACTIVE,            /* as a client, connected to a server */
+  ACTIVE_DISCONNECT  /* as a client, disconnected from server */
+};
+
+/* do-nothing macro replacement for systems which lack siginterrupt() */
+
+#ifndef HAVE_SIGINTERRUPT
+#define siginterrupt(x,y) do {} while(0)
 #endif
 
-const char *serverlogfile = (char *)DEFAULT_LOGFILE;
+/* vars used to keep around previous signal handlers */
+
+typedef RETSIGTYPE (*SIGHANDLER_T)(int);
+
+static SIGHANDLER_T old_sighup_handler  = SIG_ERR;
+static SIGHANDLER_T old_sigpipe_handler = SIG_ERR;
+static SIGHANDLER_T old_sigalrm_handler = SIG_ERR;
+static SIGHANDLER_T old_sigint_handler  = SIG_ERR;
+static SIGHANDLER_T old_sigterm_handler = SIG_ERR;
+
+/* var which if set indicates that the program should finish execution */
+
+SIG_ATOMIC_T got_exit_signal = 0;
+
+/* if next is set indicates the first signal handled in exit_signal_handler */
+
+static volatile int exit_signal = 0;
+
+/* signal handler that will be triggered to indicate that the program
+  should finish its execution in a controlled manner as soon as possible.
+  The first time this is called it will set got_exit_signal to one and
+  store in exit_signal the signal that triggered its execution. */
+
+static RETSIGTYPE exit_signal_handler(int signum)
+{
+  int old_errno = ERRNO;
+  if(got_exit_signal == 0) {
+    got_exit_signal = 1;
+    exit_signal = signum;
+  }
+  (void)signal(signum, exit_signal_handler);
+  SET_ERRNO(old_errno);
+}
+
+static void install_signal_handlers(void)
+{
+#ifdef SIGHUP
+  /* ignore SIGHUP signal */
+  if((old_sighup_handler = signal(SIGHUP, SIG_IGN)) == SIG_ERR)
+    logmsg("cannot install SIGHUP handler: 5s", strerror(ERRNO));
+#endif
+#ifdef SIGPIPE
+  /* ignore SIGPIPE signal */
+  if((old_sigpipe_handler = signal(SIGPIPE, SIG_IGN)) == SIG_ERR)
+    logmsg("cannot install SIGPIPE handler: 5s", strerror(ERRNO));
+#endif
+#ifdef SIGALRM
+  /* ignore SIGALRM signal */
+  if((old_sigalrm_handler = signal(SIGALRM, SIG_IGN)) == SIG_ERR)
+    logmsg("cannot install SIGALRM handler: 5s", strerror(ERRNO));
+#endif
+#ifdef SIGINT
+  /* handle SIGINT signal with our exit_signal_handler */
+  if((old_sigint_handler = signal(SIGINT, exit_signal_handler)) == SIG_ERR)
+    logmsg("cannot install SIGINT handler: 5s", strerror(ERRNO));
+  else
+    siginterrupt(SIGINT, 1);
+#endif
+#ifdef SIGTERM
+  /* handle SIGTERM signal with our exit_signal_handler */
+  if((old_sigterm_handler = signal(SIGTERM, exit_signal_handler)) == SIG_ERR)
+    logmsg("cannot install SIGTERM handler: 5s", strerror(ERRNO));
+  else
+    siginterrupt(SIGTERM, 1);
+#endif
+}
+
+static void restore_signal_handlers(void)
+{
+#ifdef SIGHUP
+  if(SIG_ERR != old_sighup_handler)
+    (void)signal(SIGHUP, old_sighup_handler);
+#endif
+#ifdef SIGPIPE
+  if(SIG_ERR != old_sigpipe_handler)
+    (void)signal(SIGPIPE, old_sigpipe_handler);
+#endif
+#ifdef SIGALRM
+  if(SIG_ERR != old_sigalrm_handler)
+    (void)signal(SIGALRM, old_sigalrm_handler);
+#endif
+#ifdef SIGINT
+  if(SIG_ERR != old_sigint_handler)
+    (void)signal(SIGINT, old_sigint_handler);
+#endif
+#ifdef SIGTERM
+  if(SIG_ERR != old_sigterm_handler)
+    (void)signal(SIGTERM, old_sigterm_handler);
+#endif
+}
+
+/*
+ * fullread is a wrapper around the read() function. This will repeat the call
+ * to read() until it actually has read the complete number of bytes indicated
+ * in nbytes or it fails with a condition that cannot be handled with a simple
+ * retry of the read call.
+ */
+
+static ssize_t fullread(int filedes, void *buffer, size_t nbytes)
+{
+  int error;
+  ssize_t rc;
+  ssize_t nread = 0;
+
+  do {
+    rc = read(filedes, (unsigned char *)buffer + nread, nbytes - nread);
+
+    if(got_exit_signal) {
+      logmsg("signalled to die");
+      return -1;
+    }
+
+    if(rc < 0) {
+      error = ERRNO;
+      if((error == EINTR) || (error == EAGAIN))
+        continue;
+      logmsg("unrecoverable read() failure: %s", strerror(error));
+      return -1;
+    }
+
+    if(rc == 0) {
+      logmsg("got 0 reading from stdin");
+      return 0;
+    }
+
+    nread += rc;
+
+  } while((size_t)nread < nbytes);
+
+  if(verbose)
+    logmsg("read %ld bytes", (long)nread);
+
+  return nread;
+}
+
+/*
+ * fullwrite is a wrapper around the write() function. This will repeat the
+ * call to write() until it actually has written the complete number of bytes
+ * indicated in nbytes or it fails with a condition that cannot be handled
+ * with a simple retry of the write call.
+ */
+
+static ssize_t fullwrite(int filedes, const void *buffer, size_t nbytes)
+{
+  int error;
+  ssize_t wc;
+  ssize_t nwrite = 0;
+
+  do {
+    wc = write(filedes, (unsigned char *)buffer + nwrite, nbytes - nwrite);
+
+    if(got_exit_signal) {
+      logmsg("signalled to die");
+      return -1;
+    }
+
+    if(wc < 0) {
+      error = ERRNO;
+      if((error == EINTR) || (error == EAGAIN))
+        continue;
+      logmsg("unrecoverable write() failure: %s", strerror(error));
+      return -1;
+    }
+
+    if(wc == 0) {
+      logmsg("put 0 writing to stdout");
+      return 0;
+    }
+
+    nwrite += wc;
+
+  } while((size_t)nwrite < nbytes);
+
+  if(verbose)
+    logmsg("wrote %ld bytes", (long)nwrite);
+
+  return nwrite;
+}
+
+/*
+ * read_stdin tries to read from stdin nbytes into the given buffer. This is a
+ * blocking function that will only return TRUE when nbytes have actually been
+ * read or FALSE when an unrecoverable error has been detected. Failure of this
+ * function is an indication that the sockfilt process should terminate.
+ */
+
+static bool read_stdin(void *buffer, size_t nbytes)
+{
+  ssize_t nread = fullread(fileno(stdin), buffer, nbytes);
+  if(nread != (ssize_t)nbytes) {
+    logmsg("exiting...");
+    return FALSE;
+  }
+  return TRUE;
+}
+
+/*
+ * write_stdout tries to write to stdio nbytes from the given buffer. This is a
+ * blocking function that will only return TRUE when nbytes have actually been
+ * written or FALSE when an unrecoverable error has been detected. Failure of
+ * this function is an indication that the sockfilt process should terminate.
+ */
+
+static bool write_stdout(const void *buffer, size_t nbytes)
+{
+  ssize_t nwrite = fullwrite(fileno(stdout), buffer, nbytes);
+  if(nwrite != (ssize_t)nbytes) {
+    logmsg("exiting...");
+    return FALSE;
+  }
+  return TRUE;
+}
 
 static void lograw(unsigned char *buffer, ssize_t len)
 {
@@ -135,34 +389,15 @@ static void lograw(unsigned char *buffer, ssize_t len)
     logmsg("'%s'", data);
 }
 
-#ifdef SIGPIPE
-static void sigpipe_handler(int sig)
-{
-  (void)sig; /* prevent warning */
-  sigpipe = 1;
-}
-#endif
-
-char use_ipv6=FALSE;
-unsigned short port = DEFAULT_PORT;
-unsigned short connectport = 0; /* if non-zero, we activate this mode */
-
-enum sockmode {
-  PASSIVE_LISTEN,    /* as a server waiting for connections */
-  PASSIVE_CONNECT,   /* as a server, connected to a client */
-  ACTIVE,            /* as a client, connected to a server */
-  ACTIVE_DISCONNECT  /* as a client, disconnected from server */
-};
-
 /*
   sockfdp is a pointer to an established stream or CURL_SOCKET_BAD
 
   if sockfd is CURL_SOCKET_BAD, listendfd is a listening socket we must
   accept()
 */
-static int juggle(curl_socket_t *sockfdp,
-                  curl_socket_t listenfd,
-                  enum sockmode *mode)
+static bool juggle(curl_socket_t *sockfdp,
+                   curl_socket_t listenfd,
+                   enum sockmode *mode)
 {
   struct timeval timeout;
   fd_set fds_read;
@@ -171,16 +406,30 @@ static int juggle(curl_socket_t *sockfdp,
   curl_socket_t sockfd;
   curl_socket_t maxfd;
   ssize_t rc;
-  ssize_t nread_stdin;
   ssize_t nread_socket;
   ssize_t bytes_written;
   ssize_t buffer_len;
+  int error;
 
  /* 'buffer' is this excessively large only to be able to support things like
     test 1003 which tests exceedingly large server response lines */
   unsigned char buffer[17010];
   char data[16];
 
+  if(got_exit_signal) {
+    logmsg("signalled to die, exiting...");
+    return FALSE;
+  }
+
+#ifdef HAVE_GETPPID
+  /* As a last resort, quit if sockfilt process becomes orphan. Just in case
+     parent ftpserver process has died without killing its sockfilt children */
+  if(getppid() <= 1) {
+    logmsg("process becomes orphan, exiting");
+    return FALSE;
+  }
+#endif
+
   timeout.tv_sec = 120;
   timeout.tv_usec = 0;
 
@@ -239,18 +488,28 @@ static int juggle(curl_socket_t *sockfdp,
 
   } /* switch(*mode) */
 
+
   do {
-    rc = select(maxfd + 1, &fds_read, &fds_write, &fds_err, &timeout);
-  } while((rc == -1) && (SOCKERRNO == EINTR));
 
-  switch(rc) {
-  case -1:
+    rc = select((int)maxfd + 1, &fds_read, &fds_write, &fds_err, &timeout);
+
+    if(got_exit_signal) {
+      logmsg("signalled to die, exiting...");
+      return FALSE;
+    }
+
+  } while((rc == -1) && ((error = SOCKERRNO) == EINTR));
+
+  if(rc < 0) {
+    logmsg("select() failed with error: (%d) %s",
+           error, strerror(error));
     return FALSE;
-
-  case 0: /* timeout! */
-    return TRUE;
   }
 
+  if(rc == 0)
+    /* timeout */
+    return TRUE;
+
 
   if(FD_ISSET(fileno(stdin), &fds_read)) {
     /* read from stdin, commands/data to be dealt with and possibly passed on
@@ -267,102 +526,86 @@ static int juggle(curl_socket_t *sockfdp,
 
        DATA - plain pass-thru data
     */
-    nread_stdin = read(fileno(stdin), buffer, 5);
-    if(5 == nread_stdin) {
 
-      logmsg("Received %c%c%c%c (on stdin)",
-             buffer[0], buffer[1], buffer[2], buffer[3] );
+    if(!read_stdin(buffer, 5))
+      return FALSE;
 
-      if(!memcmp("PING", buffer, 4)) {
-        /* send reply on stdout, just proving we are alive */
-        write(fileno(stdout), "PONG\n", 5);
-      }
+    logmsg("Received %c%c%c%c (on stdin)",
+           buffer[0], buffer[1], buffer[2], buffer[3] );
 
-      else if(!memcmp("PORT", buffer, 4)) {
-        /* Question asking us what PORT number we are listening to.
-           Replies to PORT with "IPv[num]/[port]" */
-        sprintf((char *)buffer, "IPv%d/%d\n", use_ipv6?6:4, (int)port);
-        buffer_len = (ssize_t)strlen((char *)buffer);
-        snprintf(data, sizeof(data), "PORT\n%04x\n", buffer_len);
-        write(fileno(stdout), data, 10);
-        write(fileno(stdout), buffer, buffer_len);
-      }
-      else if(!memcmp("QUIT", buffer, 4)) {
-        /* just die */
-        logmsg("quits");
+    if(!memcmp("PING", buffer, 4)) {
+      /* send reply on stdout, just proving we are alive */
+      if(!write_stdout("PONG\n", 5))
+        return FALSE;
+    }
+
+    else if(!memcmp("PORT", buffer, 4)) {
+      /* Question asking us what PORT number we are listening to.
+         Replies to PORT with "IPv[num]/[port]" */
+      sprintf((char *)buffer, "IPv%d/%d\n", use_ipv6?6:4, (int)port);
+      buffer_len = (ssize_t)strlen((char *)buffer);
+      snprintf(data, sizeof(data), "PORT\n%04x\n", buffer_len);
+      if(!write_stdout(data, 10))
+        return FALSE;
+      if(!write_stdout(buffer, buffer_len))
+        return FALSE;
+    }
+    else if(!memcmp("QUIT", buffer, 4)) {
+      /* just die */
+      logmsg("quits");
+      return FALSE;
+    }
+    else if(!memcmp("DATA", buffer, 4)) {
+      /* data IN => data OUT */
+
+      if(!read_stdin(buffer, 5))
+        return FALSE;
+
+      buffer[5] = '\0';
+
+      buffer_len = (ssize_t)strtol((char *)buffer, NULL, 16);
+      if (buffer_len > (ssize_t)sizeof(buffer)) {
+        logmsg("ERROR: Buffer size (%ld bytes) too small for data size "
+               "(%ld bytes)", (long)sizeof(buffer), (long)buffer_len);
         return FALSE;
       }
-      else if(!memcmp("DATA", buffer, 4)) {
-        /* data IN => data OUT */
+      logmsg("> %d bytes data, server => client", buffer_len);
 
-        if(5 != read(fileno(stdin), buffer, 5))
+      if(!read_stdin(buffer, buffer_len))
+        return FALSE;
+
+      lograw(buffer, buffer_len);
+
+      if(*mode == PASSIVE_LISTEN) {
+        logmsg("*** We are disconnected!");
+        if(!write_stdout("DISC\n", 5))
           return FALSE;
-        buffer[5] = '\0';
-
-        buffer_len = (ssize_t)strtol((char *)buffer, NULL, 16);
-        if (buffer_len > (ssize_t)sizeof(buffer)) {
-          logmsg("Buffer size %d too small for data size %d", 
-                   (int)sizeof(buffer), buffer_len);
-          return FALSE;
-        }
-        logmsg("> %d bytes data, server => client", buffer_len);
-
-        if(buffer_len > (ssize_t)sizeof(buffer)) {
-          logmsg("ERROR: %d bytes of data does not fit within the %d "
-                 "bytes buffer", buffer_len, sizeof(buffer));
-          return FALSE;
-        }
-
-        /*
-         * To properly support huge data chunks, we need to repeat the call
-         * to read() until we're done or it fails.
-         */
-        nread_stdin = 0;
-        do {
-          /* get data in the buffer at the correct position */
-          rc = read(fileno(stdin), &buffer[nread_stdin],
-                    buffer_len - nread_stdin);
-          logmsg("read %d bytes", rc);
-          if(rc <= 0)
-            return FALSE;
-          nread_stdin += rc;
-        } while (nread_stdin < buffer_len);
-
-        lograw(buffer, buffer_len);
-
-        if(*mode == PASSIVE_LISTEN) {
-          logmsg("*** We are disconnected!");
-          write(fileno(stdout), "DISC\n", 5);
-        }
-        else {
-          /* send away on the socket */
-          bytes_written = swrite(sockfd, buffer, buffer_len);
-          if(bytes_written != buffer_len) {
-            logmsg("Not all data was sent. Bytes to send: %d sent: %d", 
-                   buffer_len, bytes_written);
-          }
-        }
       }
-      else if(!memcmp("DISC", buffer, 4)) {
-        /* disconnect! */
-        write(fileno(stdout), "DISC\n", 5);
-        if(sockfd != CURL_SOCKET_BAD) {
-          logmsg("====> Client forcibly disconnected");
-          sclose(sockfd);
-          *sockfdp = CURL_SOCKET_BAD;
-          if(*mode == PASSIVE_CONNECT)
-            *mode = PASSIVE_LISTEN;
-          else
-            *mode = ACTIVE_DISCONNECT;
+      else {
+        /* send away on the socket */
+        bytes_written = swrite(sockfd, buffer, buffer_len);
+        if(bytes_written != buffer_len) {
+          logmsg("Not all data was sent. Bytes to send: %d sent: %d", 
+                 buffer_len, bytes_written);
         }
-        else
-          logmsg("attempt to close already dead connection");
-        return TRUE;
       }
     }
-    else if(-1 == nread_stdin) {
-      logmsg("read %d from stdin, exiting", nread_stdin);
-      return FALSE;
+    else if(!memcmp("DISC", buffer, 4)) {
+      /* disconnect! */
+      if(!write_stdout("DISC\n", 5))
+        return FALSE;
+      if(sockfd != CURL_SOCKET_BAD) {
+        logmsg("====> Client forcibly disconnected");
+        sclose(sockfd);
+        *sockfdp = CURL_SOCKET_BAD;
+        if(*mode == PASSIVE_CONNECT)
+          *mode = PASSIVE_LISTEN;
+        else
+          *mode = ACTIVE_DISCONNECT;
+      }
+      else
+        logmsg("attempt to close already dead connection");
+      return TRUE;
     }
   }
 
@@ -377,7 +620,8 @@ static int juggle(curl_socket_t *sockfdp,
         logmsg("accept() failed");
       else {
         logmsg("====> Client connect");
-        write(fileno(stdout), "CNCT\n", 5);
+        if(!write_stdout("CNCT\n", 5))
+          return FALSE;
         *sockfdp = sockfd; /* store the new socket */
         *mode = PASSIVE_CONNECT; /* we have connected */
       }
@@ -389,7 +633,8 @@ static int juggle(curl_socket_t *sockfdp,
 
     if(nread_socket <= 0) {
       logmsg("====> Client disconnect");
-      write(fileno(stdout), "DISC\n", 5);
+      if(!write_stdout("DISC\n", 5))
+        return FALSE;
       sclose(sockfd);
       *sockfdp = CURL_SOCKET_BAD;
       if(*mode == PASSIVE_CONNECT)
@@ -400,8 +645,10 @@ static int juggle(curl_socket_t *sockfdp,
     }
 
     snprintf(data, sizeof(data), "DATA\n%04x\n", nread_socket);
-    write(fileno(stdout), data, 10);
-    write(fileno(stdout), buffer, nread_socket);
+    if(!write_stdout(data, 10))
+      return FALSE;
+    if(!write_stdout(buffer, nread_socket))
+      return FALSE;
 
     logmsg("< %d bytes data, client => server", nread_socket);
     lograw(buffer, nread_socket);
@@ -420,25 +667,48 @@ static curl_socket_t sockdaemon(curl_socket_t sock,
 #endif /* ENABLE_IPV6 */
   int flag = 1;
   int rc;
-  int maxretr = 12;
-  int delay= 10;
+  int totdelay = 0;
+  int maxretr = 10;
+  int delay= 20;
+  int attempt = 0;
+  int error = 0;
 
-  rc = setsockopt(sock, SOL_SOCKET, SO_REUSEADDR,
-       (void *)&flag, sizeof(flag));
-  while ((rc < 0) && maxretr) {
-    maxretr--;
-    go_sleep(delay);
-    delay *= 2; /* double the sleep for next attempt */
+  do {
+    attempt++;
     rc = setsockopt(sock, SOL_SOCKET, SO_REUSEADDR,
          (void *)&flag, sizeof(flag));
-  }
-  if (rc < 0) {
-    perror("setsockopt(SO_REUSEADDR)");
+    if(rc) {
+      error = SOCKERRNO;
+      if(maxretr) {
+        rc = wait_ms(delay);
+        if(rc) {
+          /* should not happen */
+          error = SOCKERRNO;
+          logmsg("wait_ms() failed: (%d) %s", error, strerror(error));
+          sclose(sock);
+          return CURL_SOCKET_BAD;
+        }
+        if(got_exit_signal) {
+          logmsg("signalled to die, exiting...");
+          sclose(sock);
+          return CURL_SOCKET_BAD;
+        }
+        totdelay += delay;
+        delay *= 2; /* double the sleep for next attempt */
+      }
+    }
+  } while(rc && maxretr--);
+
+  if(rc) {
+    logmsg("setsockopt(SO_REUSEADDR) failed %d times in %d ms. Error: (%d) %s",
+           attempt, totdelay, error, strerror(error));
+    logmsg("Continuing anyway...");
   }
 
 #ifdef ENABLE_IPV6
   if(!use_ipv6) {
 #endif
+    memset(&me, 0, sizeof(me));
     me.sin_family = AF_INET;
     me.sin_addr.s_addr = INADDR_ANY;
     me.sin_port = htons(*listenport);
@@ -446,16 +716,17 @@ static curl_socket_t sockdaemon(curl_socket_t sock,
 #ifdef ENABLE_IPV6
   }
   else {
-    memset(&me6, 0, sizeof(struct sockaddr_in6));
+    memset(&me6, 0, sizeof(me6));
     me6.sin6_family = AF_INET6;
     me6.sin6_addr = in6addr_any;
     me6.sin6_port = htons(*listenport);
     rc = bind(sock, (struct sockaddr *) &me6, sizeof(me6));
   }
 #endif /* ENABLE_IPV6 */
-  if(rc < 0) {
-    perror("binding stream socket");
-    logmsg("Error binding socket");
+  if(rc) {
+    error = SOCKERRNO;
+    logmsg("Error binding socket: (%d) %s", error, strerror(error));
+    sclose(sock);
     return CURL_SOCKET_BAD;
   }
 
@@ -468,16 +739,21 @@ static curl_socket_t sockdaemon(curl_socket_t sock,
 
     if(getsockname(sock, (struct sockaddr *) &add,
                    &socksize)<0) {
-      logmsg("getsockname() failed with error: %d", SOCKERRNO);
+      error = SOCKERRNO;
+      logmsg("getsockname() failed with error: (%d) %s",
+             error, strerror(error));
+      sclose(sock);
       return CURL_SOCKET_BAD;
     }
     *listenport = ntohs(add.sin_port);
   }
 
   /* start accepting connections */
-  rc = listen(sock, 4);
+  rc = listen(sock, 5);
   if(0 != rc) {
-    logmsg("listen() failed with error: %d", SOCKERRNO);
+    error = SOCKERRNO;
+    logmsg("listen() failed with error: (%d) %s",
+           error, strerror(error));
     sclose(sock);
     return CURL_SOCKET_BAD;
   }
@@ -485,28 +761,6 @@ static curl_socket_t sockdaemon(curl_socket_t sock,
   return sock;
 }
 
-static curl_socket_t mksock(bool ipv6)
-{
-  curl_socket_t sock;
-#ifdef ENABLE_IPV6
-  if(!ipv6)
-#else
-    (void)ipv6;
-#endif
-  sock = socket(AF_INET, SOCK_STREAM, 0);
-#ifdef ENABLE_IPV6
-  else
-    sock = socket(AF_INET6, SOCK_STREAM, 0);
-#endif
-
-  if (CURL_SOCKET_BAD == sock) {
-    perror("opening stream socket");
-    logmsg("Error opening socket");
-  }
-
-  return sock;
-}
-
 
 int main(int argc, char *argv[])
 {
@@ -514,9 +768,9 @@ int main(int argc, char *argv[])
 #ifdef ENABLE_IPV6
   struct sockaddr_in6 me6;
 #endif /* ENABLE_IPV6 */
-  curl_socket_t sock;
-  curl_socket_t msgsock;
-  FILE *pidfile;
+  curl_socket_t sock = CURL_SOCKET_BAD;
+  curl_socket_t msgsock = CURL_SOCKET_BAD;
+  int wrotepidfile = 0;
   char *pidname= (char *)".sockfilt.pid";
   int rc;
   int error;
@@ -535,6 +789,10 @@ int main(int argc, char *argv[])
              );
       return 0;
     }
+    else if(!strcmp("--verbose", argv[arg])) {
+      verbose = TRUE;
+      arg++;
+    }
     else if(!strcmp("--pidfile", argv[arg])) {
       arg++;
       if(argc>arg)
@@ -583,6 +841,7 @@ int main(int argc, char *argv[])
     else {
       puts("Usage: sockfilt [option]\n"
            " --version\n"
+           " --verbose\n"
            " --logfile [file]\n"
            " --pidfile [file]\n"
            " --ipv4\n"
@@ -597,23 +856,24 @@ int main(int argc, char *argv[])
 #ifdef WIN32
   win32_init();
   atexit(win32_cleanup);
-#else
-
-#ifdef SIGPIPE
-#ifdef HAVE_SIGNAL
-  signal(SIGPIPE, sigpipe_handler);
-#endif
-#ifdef HAVE_SIGINTERRUPT
-  siginterrupt(SIGPIPE, 1);
-#endif
-#endif
 #endif
 
+  install_signal_handlers();
 
-  sock = mksock(use_ipv6);
-  if (CURL_SOCKET_BAD == sock) {
-    logmsg("Error opening socket: %d", SOCKERRNO);
-    return 1;
+#ifdef ENABLE_IPV6
+  if(!use_ipv6)
+#endif
+    sock = socket(AF_INET, SOCK_STREAM, 0);
+#ifdef ENABLE_IPV6
+  else
+    sock = socket(AF_INET6, SOCK_STREAM, 0);
+#endif
+
+  if(CURL_SOCKET_BAD == sock) {
+    error = SOCKERRNO;
+    logmsg("Error creating socket: (%d) %s",
+           error, strerror(error));
+    goto sockfilt_cleanup;
   }
 
   if(connectport) {
@@ -645,10 +905,10 @@ int main(int argc, char *argv[])
     }
 #endif /* ENABLE_IPV6 */
     if(rc) {
-      perror("connecting stream socket");
-      logmsg("Error connecting to port %d", port);
-      sclose(sock);
-      return 1;
+      error = SOCKERRNO;
+      logmsg("Error connecting to port %d: (%d) %s",
+             connectport, error, strerror(error));
+      goto sockfilt_cleanup;
     }
     logmsg("====> Client connect");
     msgsock = sock; /* use this as stream */
@@ -657,7 +917,7 @@ int main(int argc, char *argv[])
     /* passive daemon style */
     sock = sockdaemon(sock, &port);
     if(CURL_SOCKET_BAD == sock)
-      return 1;
+      goto sockfilt_cleanup;
     msgsock = CURL_SOCKET_BAD; /* no stream socket yet */
   }
 
@@ -669,26 +929,36 @@ int main(int argc, char *argv[])
   else
     logmsg("Listening on port %d", port);
 
-  pidfile = fopen(pidname, "w");
-  if(pidfile) {
-    int pid = (int)getpid();
-    fprintf(pidfile, "%d\n", pid);
-    fclose(pidfile);
-    logmsg("Wrote pid %d to %s", pid, pidname);
-  }
-  else {
-    error = ERRNO;
-    logmsg("fopen() failed with error: %d %s", error, strerror(error));
-    logmsg("Error opening file: %s", pidname);
-    logmsg("Couldn't write pid file");
-    sclose(sock);
-    return 1;
-  }
+  wrotepidfile = write_pidfile(pidname);
+  if(!wrotepidfile)
+    goto sockfilt_cleanup;
 
   while(juggle(&msgsock, sock, &mode));
 
-  sclose(sock);
+sockfilt_cleanup:
 
+  if((msgsock != sock) && (msgsock != CURL_SOCKET_BAD))
+    sclose(msgsock);
+
+  if(sock != CURL_SOCKET_BAD)
+    sclose(sock);
+
+  if(wrotepidfile)
+    unlink(pidname);
+
+  restore_signal_handlers();
+
+  if(got_exit_signal) {
+    logmsg("============> sockfilt exits with signal (%d)", exit_signal);
+    /*
+     * To properly set the return status of the process we
+     * must raise the same signal SIGINT or SIGTERM that we
+     * caught and let the old handler take care of it.
+     */
+    raise(exit_signal);
+  }
+
+  logmsg("============> sockfilt quits");
   return 0;
 }
 
