@@ -5,7 +5,7 @@
  *                            | (__| |_| |  _ <| |___
  *                             \___|\___/|_| \_\_____|
  *
- * Copyright (C) 1998 - 2007, Daniel Stenberg, <daniel@haxx.se>, et al.
+ * Copyright (C) 1998 - 2008, Daniel Stenberg, <daniel@haxx.se>, et al.
  *
  * This software is licensed as described in the file COPYING, which
  * you should have received as part of this distribution. The terms
@@ -18,7 +18,7 @@
  * This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY
  * KIND, either express or implied.
  *
- * $Id: sws.c,v 1.110 2008-01-25 05:08:53 yangtse Exp $
+ * $Id: sws.c,v 1.120 2008-04-23 23:55:34 yangtse Exp $
  ***************************************************************************/
 
 /* sws.c: simple (silly?) web server
@@ -29,16 +29,9 @@
  */
 #include "setup.h" /* portability help from the lib directory */
 
-#include <stdio.h>
-#include <stdlib.h>
-#include <string.h>
-#include <stdarg.h>
+#ifdef HAVE_SIGNAL_H
 #include <signal.h>
-#include <time.h>
-#include <sys/time.h>
-#include <sys/types.h>
-#include <ctype.h>
-
+#endif
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #endif
@@ -83,9 +76,9 @@
 
 long prevtestno=-1; /* previous test number we served */
 long prevpartno=-1; /* previous part number we served */
-bool prevbounce;    /* instructs the server to increase the part number for
-                       a test in case the identical testno+partno request
-                       shows up again */
+bool prevbounce=FALSE; /* instructs the server to increase the part number for
+                          a test in case the identical testno+partno request
+                          shows up again */
 
 #define RCMD_NORMALREQ 0 /* default request, use the tests file normally */
 #define RCMD_IDLE      1 /* told to sit idle */
@@ -97,7 +90,7 @@ struct httprequest {
   int offset;     /* size of the incoming request */
   long testno;     /* test number found in the request */
   long partno;     /* part number found in the request */
-  int open;       /* keep connection open info, as found in the request */
+  bool open;      /* keep connection open info, as found in the request */
   bool auth_req;  /* authentication required, don't wait for body unless
                      there's an Authorization header */
   bool auth;      /* Authorization header present in the incoming request */
@@ -132,6 +125,7 @@ const char *serverlogfile = DEFAULT_LOGFILE;
 #define MAXDOCNAMELEN_TXT "139999"
 
 #define REQUEST_KEYWORD_SIZE 256
+#define REQUEST_KEYWORD_SIZE_TXT "255"
 
 #define CMD_AUTH_REQUIRED "auth_required"
 
@@ -196,7 +190,7 @@ static void sigpipe_handler(int sig)
 int ProcessRequest(struct httprequest *req)
 {
   char *line=&req->reqbuf[req->checkindex];
-  char chunked=FALSE;
+  bool chunked = FALSE;
   static char request[REQUEST_KEYWORD_SIZE];
   static char doc[MAXDOCNAMELEN];
   char logbuf[256];
@@ -210,7 +204,8 @@ int ProcessRequest(struct httprequest *req)
   /* try to figure out the request characteristics as soon as possible, but
      only once! */
   if((req->testno == DOCNUMBER_NOTHING) &&
-     sscanf(line, "%" REQBUFSIZ_TXT"s %" MAXDOCNAMELEN_TXT "s HTTP/%d.%d",
+     sscanf(line,
+            "%" REQUEST_KEYWORD_SIZE_TXT"s %" MAXDOCNAMELEN_TXT "s HTTP/%d.%d",
             request,
             doc,
             &prot_major,
@@ -475,6 +470,7 @@ int ProcessRequest(struct httprequest *req)
 /* store the entire request in a file */
 void storerequest(char *reqbuf, ssize_t totalsize)
 {
+  int res;
   int error;
   ssize_t written;
   ssize_t writeleft;
@@ -509,62 +505,74 @@ void storerequest(char *reqbuf, ssize_t totalsize)
       writeleft -= written;
   } while ((writeleft > 0) && ((error = ERRNO) == EINTR));
 
-  fclose(dump);  /* close it ASAP */
-
   if (writeleft > 0) {
     logmsg("Error writing file %s error: %d %s",
            REQUEST_DUMP, error, strerror(error));
     logmsg("Wrote only (%d bytes) of (%d bytes) request input to %s",
            totalsize-writeleft, totalsize, REQUEST_DUMP);
   }
-  else {
+
+  do {
+    res = fclose(dump);
+  } while(res && ((error = ERRNO) == EINTR));
+  if(res)
+    logmsg("Error closing file %s error: %d %s",
+           REQUEST_DUMP, error, strerror(error));
+
+  if(!writeleft)
     logmsg("Wrote request (%d bytes) input to " REQUEST_DUMP,
            totalsize);
-  }
 }
 
 /* return 0 on success, non-zero on failure */
 static int get_request(curl_socket_t sock, struct httprequest *req)
 {
-  int fail= FALSE;
+  int fail = 0;
   char *reqbuf = req->reqbuf;
+  ssize_t got = 0;
+
+  char *pipereq;
+  int pipereq_length = 0;
 
-  char pipereq[REQBUFSIZ];
-  int pipereq_length;
   if(req->pipelining) {
+    pipereq = reqbuf + req->checkindex;
     pipereq_length = req->offset - req->checkindex;
-    memcpy(pipereq, reqbuf + req->checkindex, pipereq_length);
   }
-  else
-    pipereq_length = 0;
 
   /*** Init the httpreqest structure properly for the upcoming request ***/
-  memset(req, 0, sizeof(struct httprequest));
 
-  /* here's what should not be 0 from the start */
-  req->testno = DOCNUMBER_NOTHING; /* safe default */
-  req->open = TRUE; /* connection should remain open and wait for more
-                       commands */
+  req->checkindex = 0;
+  req->offset = 0;
+  req->testno = DOCNUMBER_NOTHING;
+  req->partno = 0;
+  req->open = TRUE;
+  req->auth_req = FALSE;
+  req->auth = FALSE;
+  req->cl = 0;
+  req->digest = FALSE;
+  req->ntlm = FALSE;
   req->pipe = 0;
+  req->rcmd = RCMD_NORMALREQ;
+  req->prot_version = 0;
+  req->pipelining = FALSE;
 
   /*** end of httprequest init ***/
 
-  while (req->offset < REQBUFSIZ) {
-    ssize_t got;
+  while (req->offset < REQBUFSIZ-1) {
     if(pipereq_length) {
-      memcpy(reqbuf, pipereq, pipereq_length);
+      memmove(reqbuf, pipereq, pipereq_length); 
       got = pipereq_length;
       pipereq_length = 0;
     }
     else
-      got = sread(sock, reqbuf + req->offset, REQBUFSIZ - req->offset);
+      got = sread(sock, reqbuf + req->offset, REQBUFSIZ-1 - req->offset);
     if (got <= 0) {
       if (got < 0) {
         logmsg("recv() returned error: %d", SOCKERRNO);
         return DOCNUMBER_INTERNAL;
       }
       logmsg("Connection closed by client");
-      reqbuf[req->offset]=0;
+      reqbuf[req->offset] = '\0';
 
       /* dump the request receivied so far to the external file */
       storerequest(reqbuf, req->offset);
@@ -574,8 +582,7 @@ static int get_request(curl_socket_t sock, struct httprequest *req)
     logmsg("Read %d bytes", got);
 
     req->offset += got;
-
-    reqbuf[req->offset] = 0;
+    reqbuf[req->offset] = '\0';
 
     if(ProcessRequest(req)) {
       if(req->pipe--) {
@@ -586,19 +593,25 @@ static int get_request(curl_socket_t sock, struct httprequest *req)
     }
   }
 
-  if (req->offset >= REQBUFSIZ) {
+  if((req->offset == REQBUFSIZ-1) && (got > 0)) {
+    logmsg("Request would overflow buffer, closing connection");
+    /* dump request received so far to external file anyway */
+    reqbuf[REQBUFSIZ-1] = '\0';
+    fail = 1;
+  }
+  else if(req->offset > REQBUFSIZ-1) {
     logmsg("Request buffer overflow, closing connection");
-    reqbuf[REQBUFSIZ-1]=0;
-    fail = TRUE;
-    /* dump the request to an external file anyway */
+    /* dump request received so far to external file anyway */
+    reqbuf[REQBUFSIZ-1] = '\0';
+    fail = 1;
   }
   else
-    reqbuf[req->offset]=0;
+    reqbuf[req->offset] = '\0';
 
   /* dump the request to an external file */
   storerequest(reqbuf, req->pipelining ? req->checkindex : req->offset);
 
-  return fail; /* success */
+  return fail; /* return 0 on success */
 }
 
 /* returns -1 on failure */
@@ -607,14 +620,16 @@ static int send_doc(curl_socket_t sock, struct httprequest *req)
   ssize_t written;
   size_t count;
   const char *buffer;
-  char *ptr;
+  char *ptr=NULL;
   FILE *stream;
   char *cmd=NULL;
   size_t cmdsize=0;
   FILE *dump;
-  int persistant = TRUE;
+  bool persistant = TRUE;
+  bool sendfailure = FALSE;
   size_t responsesize;
   int error;
+  int res;
 
   static char weare[256];
 
@@ -656,7 +671,7 @@ static int send_doc(curl_socket_t sock, struct httprequest *req)
     case DOCNUMBER_WERULEZ:
       /* we got a "friends?" question, reply back that we sure are */
       logmsg("Identifying ourselves as friends");
-      sprintf(msgbuf, "WE ROOLZ: %d\r\n", (int)getpid());
+      sprintf(msgbuf, "WE ROOLZ: %ld\r\n", (long)getpid());
       msglen = strlen(msgbuf);
       sprintf(weare, "HTTP/1.1 200 OK\r\nContent-Length: %d\r\n\r\n%s",
               msglen, msgbuf);
@@ -754,9 +769,8 @@ static int send_doc(curl_socket_t sock, struct httprequest *req)
       num = 200;
     written = swrite(sock, buffer, num);
     if (written < 0) {
-      fclose(dump);
-      logmsg("Sending response failed and we bailed out!");
-      return -1;
+      sendfailure = TRUE;
+      break;
     }
     else {
       logmsg("Sent off %d bytes", written);
@@ -768,7 +782,22 @@ static int send_doc(curl_socket_t sock, struct httprequest *req)
     buffer += written;
   } while(count>0);
 
-  fclose(dump);
+  do {
+    res = fclose(dump);
+  } while(res && ((error = ERRNO) == EINTR));
+  if(res)
+    logmsg("Error closing file %s error: %d %s",
+           RESPONSE_DUMP, error, strerror(error));
+
+  if(sendfailure) {
+    logmsg("Sending response failed. Only (%d bytes) of (%d bytes) were sent",
+           responsesize-count, responsesize);
+    if(ptr)
+      free(ptr);
+    if(cmd)
+      free(cmd);
+    return -1;
+  }
 
   logmsg("Response sent (%d bytes) and written to " RESPONSE_DUMP,
          responsesize);
@@ -807,7 +836,7 @@ static int send_doc(curl_socket_t sock, struct httprequest *req)
   return 0;
 }
 
-char use_ipv6=FALSE;
+bool use_ipv6=FALSE;
 
 int main(int argc, char *argv[])
 {
@@ -818,11 +847,9 @@ int main(int argc, char *argv[])
   curl_socket_t sock, msgsock;
   int flag;
   unsigned short port = DEFAULT_PORT;
-  FILE *pidfile;
   char *pidname= (char *)".http.pid";
   struct httprequest req;
   int rc;
-  int error;
   int arg=1;
 #ifdef CURL_SWS_FORK_ENABLED
   bool use_fork = FALSE;
@@ -932,16 +959,7 @@ int main(int argc, char *argv[])
     return 1;
   }
 
-  pidfile = fopen(pidname, "w");
-  if(pidfile) {
-    fprintf(pidfile, "%d\n", (int)getpid());
-    fclose(pidfile);
-  }
-  else {
-    error = ERRNO;
-    logmsg("fopen() failed with error: %d %s", error, strerror(error));
-    logmsg("Error opening file: %s", pidname);
-    logmsg("Couldn't write pid file");
+  if(!write_pidfile(pidname)) {
     sclose(sock);
     return 1;
   }
@@ -970,6 +988,8 @@ int main(int argc, char *argv[])
       break;
     }
 
+    set_advisor_read_lock(SERVERLOGS_LOCK);
+
 #ifdef CURL_SWS_FORK_ENABLED
     if(use_fork) {
       /* The fork enabled version just forks off the child and don't care
@@ -1001,17 +1021,13 @@ int main(int argc, char *argv[])
     }
 #endif
 
-  /* full initialization for new request after connection */
-  memset(&req, 0, sizeof(req));
-  req.testno = DOCNUMBER_NOTHING;
-  req.open = TRUE;
-  req.auth_req = FALSE;
-  req.auth = FALSE;
-  req.digest = FALSE;
-  req.ntlm = FALSE;
-  req.pipelining = FALSE;
+    /* initialization of httprequest struct is done in get_request(), but due
+       to pipelining treatment the pipelining struct field must be initialized
+       previously to FALSE every time a new connection arrives. */
 
-  do {
+    req.pipelining = FALSE;
+
+    do {
       if(get_request(msgsock, &req))
         /* non-zero means error, break out of loop */
         break;
@@ -1023,6 +1039,11 @@ int main(int argc, char *argv[])
           req.partno++;
           logmsg("BOUNCE part number to %ld", req.partno);
         }
+        else {
+          prevbounce = FALSE;
+          prevtestno = -1;
+          prevpartno = -1;
+        }
       }
 
       send_doc(msgsock, &req);
@@ -1044,6 +1065,8 @@ int main(int argc, char *argv[])
     logmsg("====> Client disconnect");
     sclose(msgsock);
 
+    clear_advisor_read_lock(SERVERLOGS_LOCK);
+
     if (req.testno == DOCNUMBER_QUIT)
       break;
 #ifdef CURL_SWS_FORK_ENABLED
@@ -1053,6 +1076,8 @@ int main(int argc, char *argv[])
 
   sclose(sock);
 
+  clear_advisor_read_lock(SERVERLOGS_LOCK);
+
   return 0;
 }
 
