@@ -5,7 +5,7 @@
  *                            | (__| |_| |  _ <| |___
  *                             \___|\___/|_| \_\_____|
  *
- * $Id: tftpd.c,v 1.30 2007-11-15 13:20:18 yangtse Exp $
+ * $Id: tftpd.c,v 1.35 2008-04-23 23:55:34 yangtse Exp $
  *
  * Trivial file transfer protocol server.
  *
@@ -49,18 +49,13 @@
  */
 
 #include "setup.h" /* portability help from the lib directory */
-#ifdef HAVE_SYS_TYPES_H
-#include <sys/types.h>
-#endif
+
 #ifdef HAVE_SYS_IOCTL_H
 #include <sys/ioctl.h>
 #endif
-#ifdef HAVE_SYS_STAT_H
-#include <sys/stat.h>
-#endif
-
+#ifdef HAVE_SIGNAL_H
 #include <signal.h>
-
+#endif
 #ifdef HAVE_FCNTL_H
 #include <fcntl.h>
 #endif
@@ -84,12 +79,9 @@
 #endif
 
 #include <setjmp.h>
-#include <stdio.h>
-#include <errno.h>
-#include <ctype.h>
-#include <string.h>
-#include <stdlib.h>
+#ifdef HAVE_UNISTD_H
 #include <unistd.h>
+#endif
 #ifdef HAVE_PWD_H
 #include <pwd.h>
 #endif
@@ -417,14 +409,13 @@ int main(int argc, char **argv)
   struct tftphdr *tp;
   int n = 0;
   int arg = 1;
-  FILE *pidfile;
   char *pidname= (char *)".tftpd.pid";
   unsigned short port = DEFAULT_PORT;
   curl_socket_t sock;
   int flag;
   int rc;
-  int error;
   struct testcase test;
+  int result = 0;
 
   while(argc>arg) {
     if(!strcmp("--version", argv[arg])) {
@@ -505,19 +496,13 @@ int main(int argc, char **argv)
   if(rc < 0) {
     perror("binding stream socket");
     logmsg("Error binding socket");
+    sclose(sock);
     return 1;
   }
 
-  pidfile = fopen(pidname, "w");
-  if(pidfile) {
-    fprintf(pidfile, "%d\n", (int)getpid());
-    fclose(pidfile);
-  }
-  else {
-    error = ERRNO;
-    logmsg("fopen() failed with error: %d %s", error, strerror(error));
-    logmsg("Error opening file: %s", pidname);
-    logmsg("Couldn't write pid file");
+  if(!write_pidfile(pidname)) {
+    sclose(sock);
+    return 1;
   }
 
   logmsg("Running IPv%d version on port UDP/%d",
@@ -534,20 +519,25 @@ int main(int argc, char **argv)
                  (struct sockaddr *)&from, &fromlen);
     if (n < 0) {
       logmsg("recvfrom:\n");
-      return 3;
+      result = 3;
+      break;
     }
 
+    set_advisor_read_lock(SERVERLOGS_LOCK);
+
     from.sin_family = AF_INET;
 
     peer = socket(AF_INET, SOCK_DGRAM, 0);
     if (peer < 0) {
       logmsg("socket:\n");
-      return 2;
+      result = 2;
+      break;
     }
 
     if (connect(peer, (struct sockaddr *)&from, sizeof(from)) < 0) {
       logmsg("connect: fail\n");
-      return 1;
+      result = 1;
+      break;
     }
     maxtimeout = 5*TIMEOUT;
 
@@ -561,8 +551,14 @@ int main(int argc, char **argv)
         free(test.buffer);
     }
     sclose(peer);
+
+    clear_advisor_read_lock(SERVERLOGS_LOCK);
+
   } while(1);
-  return 0;
+
+  clear_advisor_read_lock(SERVERLOGS_LOCK);
+
+  return result;
 }
 
 struct formats {
@@ -660,9 +656,9 @@ static int validate_access(struct testcase *test,
 
   logmsg("trying to get file: %s mode %x", filename, mode);
 
-  if(!strncmp("verifiedserver", filename, 15)) {
+  if(!strncmp("verifiedserver", filename, 14)) {
     char weare[128];
-    size_t count = sprintf(weare, "WE ROOLZ: %d\r\n", (int)getpid());
+    size_t count = sprintf(weare, "WE ROOLZ: %ld\r\n", (long)getpid());
 
     logmsg("Are-we-friendly question received");
     test->buffer = strdup(weare);
@@ -750,8 +746,10 @@ static void timer(int signum)
   logmsg("alarm!");
 
   timeout += rexmtval;
-  if (timeout >= maxtimeout)
+  if(timeout >= maxtimeout) {
+    clear_advisor_read_lock(SERVERLOGS_LOCK);
     exit(1);
+  }
 #ifdef HAVE_SIGSETJMP
   siglongjmp(timeoutbuf, 1);
 #endif
