@@ -19,7 +19,7 @@
 # This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY
 # KIND, either express or implied.
 #
-# $Id: sshserver.pl,v 1.44 2008-01-27 02:35:20 yangtse Exp $
+# $Id: sshserver.pl,v 1.50 2008-04-04 14:47:32 yangtse Exp $
 #***************************************************************************
 
 # Starts sshd for use in the SCP, SFTP and SOCKS curl test harness tests.
@@ -44,24 +44,31 @@ use Cwd;
 use sshhelp qw(
     $sshdexe
     $sshexe
+    $sftpsrvexe
     $sftpexe
     $sshkeygenexe
     $sshdconfig
     $sshconfig
+    $sftpconfig
     $knownhosts
     $sshdlog
     $sshlog
+    $sftplog
+    $sftpcmds
     $hstprvkeyf
     $hstpubkeyf
     $cliprvkeyf
     $clipubkeyf
     display_sshdconfig
     display_sshconfig
+    display_sftpconfig
     display_sshdlog
     display_sshlog
+    display_sftplog
     dump_array
     find_sshd
     find_ssh
+    find_sftpsrv
     find_sftp
     find_sshkeygen
     logmsg
@@ -193,12 +200,23 @@ if((($sshdid =~ /OpenSSH/) && ($sshdvernum < 299)) ||
 #***************************************************************************
 # Find out sftp server plugin canonical file name
 #
+my $sftpsrv = find_sftpsrv();
+if(!$sftpsrv) {
+    logmsg "cannot find $sftpsrvexe";
+    exit 1;
+}
+logmsg "sftp server plugin found $sftpsrv" if($verbose);
+
+
+#***************************************************************************
+# Find out sftp client canonical file name
+#
 my $sftp = find_sftp();
 if(!$sftp) {
     logmsg "cannot find $sftpexe";
     exit 1;
 }
-logmsg "sftp server plugin found $sftp" if($verbose);
+logmsg "sftp client found $sftp" if($verbose);
 
 
 #***************************************************************************
@@ -311,6 +329,7 @@ if((! -e $hstprvkeyf) || (! -e $hstpubkeyf) ||
 #  AllowTcpForwarding               : OpenSSH 2.3.0 and later
 #  AllowUsers                       : OpenSSH 1.2.1 and later
 #  AuthorizedKeysFile               : OpenSSH 2.9.9 and later
+#  AuthorizedKeysFile2              : OpenSSH 2.9.9 and later
 #  Banner                           : OpenSSH 2.5.0 and later
 #  ChallengeResponseAuthentication  : OpenSSH 2.5.0 and later
 #  Ciphers                          : OpenSSH 2.1.0 and later [3]
@@ -323,11 +342,16 @@ if((! -e $hstprvkeyf) || (! -e $hstpubkeyf) ||
 #  GatewayPorts                     : OpenSSH 2.1.0 and later
 #  GSSAPIAuthentication             : OpenSSH 3.7.0 and later [1]
 #  GSSAPICleanupCredentials         : OpenSSH 3.8.0 and later [1]
+#  GSSAPIKeyExchange                :  SunSSH 1.0.0 and later [1]
+#  GSSAPIStoreDelegatedCredentials  :  SunSSH 1.0.0 and later [1]
+#  GSSCleanupCreds                  :  SunSSH 1.0.0 and later [1]
+#  GSSUseSessionCredCache           :  SunSSH 1.0.0 and later [1]
 #  HostbasedAuthentication          : OpenSSH 2.9.0 and later
 #  HostbasedUsesNameFromPacketOnly  : OpenSSH 2.9.0 and later
 #  HostKey                          : OpenSSH 1.2.1 and later
 #  IgnoreRhosts                     : OpenSSH 1.2.1 and later
 #  IgnoreUserKnownHosts             : OpenSSH 1.2.1 and later
+#  KbdInteractiveAuthentication     : OpenSSH 2.3.0 and later
 #  KeepAlive                        : OpenSSH 1.2.1 and later
 #  KerberosAuthentication           : OpenSSH 1.2.1 and later [1]
 #  KerberosGetAFSToken              : OpenSSH 3.8.0 and later [1]
@@ -338,11 +362,12 @@ if((! -e $hstprvkeyf) || (! -e $hstpubkeyf) ||
 #  ListenAddress                    : OpenSSH 1.2.1 and later
 #  LoginGraceTime                   : OpenSSH 1.2.1 and later
 #  LogLevel                         : OpenSSH 1.2.1 and later
+#  LookupClientHostnames            :  SunSSH 1.0.0 and later
 #  MACs                             : OpenSSH 2.5.0 and later [3]
 #  Match                            : OpenSSH 4.4.0 and later [3]
 #  MaxAuthTries                     : OpenSSH 3.9.0 and later
 #  MaxStartups                      : OpenSSH 2.2.0 and later
-#  PAMAuthenticationViaKbdInt
+#  PAMAuthenticationViaKbdInt       : OpenSSH 2.9.0 and later [2]
 #  PasswordAuthentication           : OpenSSH 1.2.1 and later
 #  PermitEmptyPasswords             : OpenSSH 1.2.1 and later
 #  PermitOpen                       : OpenSSH 4.4.0 and later [3]
@@ -355,6 +380,7 @@ if((! -e $hstprvkeyf) || (! -e $hstpubkeyf) ||
 #  PrintMotd                        : OpenSSH 1.2.1 and later
 #  Protocol                         : OpenSSH 2.1.0 and later
 #  PubkeyAuthentication             : OpenSSH 2.5.0 and later
+#  RhostsAuthentication             : OpenSSH 1.2.1 and later
 #  RhostsRSAAuthentication          : OpenSSH 1.2.1 and later
 #  RSAAuthentication                : OpenSSH 1.2.1 and later
 #  ServerKeyBits                    : OpenSSH 1.2.1 and later
@@ -367,6 +393,7 @@ if((! -e $hstprvkeyf) || (! -e $hstpubkeyf) ||
 #  UseLogin                         : OpenSSH 1.2.1 and later
 #  UsePAM                           : OpenSSH 3.7.0 and later [1][2]
 #  UsePrivilegeSeparation           : OpenSSH 3.2.2 and later
+#  VerifyReverseMapping             : OpenSSH 3.1.0 and later
 #  X11DisplayOffset                 : OpenSSH 1.2.1 and later [3]
 #  X11Forwarding                    : OpenSSH 1.2.1 and later
 #  X11UseLocalhost                  : OpenSSH 3.1.0 and later
@@ -391,6 +418,7 @@ push @cfgarr, 'DenyGroups';
 push @cfgarr, 'AllowGroups';
 push @cfgarr, '#';
 push @cfgarr, "AuthorizedKeysFile $path/$clipubkeyf";
+push @cfgarr, "AuthorizedKeysFile2 $path/$clipubkeyf";
 push @cfgarr, "HostKey $path/$hstprvkeyf";
 push @cfgarr, "PidFile $path/.ssh.pid";
 push @cfgarr, '#';
@@ -422,7 +450,7 @@ push @cfgarr, 'RhostsRSAAuthentication no';
 push @cfgarr, 'RSAAuthentication no';
 push @cfgarr, 'ServerKeyBits 768';
 push @cfgarr, 'StrictModes no';
-push @cfgarr, "Subsystem sftp $sftp";
+push @cfgarr, "Subsystem sftp $sftpsrv";
 push @cfgarr, 'SyslogFacility AUTH';
 push @cfgarr, 'UseLogin no';
 push @cfgarr, 'X11Forwarding no';
@@ -516,6 +544,18 @@ if(sshd_supports_opt('GSSAPIAuthentication','no')) {
 if(sshd_supports_opt('GSSAPICleanupCredentials','yes')) {
     push @cfgarr, 'GSSAPICleanupCredentials yes';
 }
+if(sshd_supports_opt('GSSAPIKeyExchange','no')) {
+    push @cfgarr, 'GSSAPIKeyExchange no';
+}
+if(sshd_supports_opt('GSSAPIStoreDelegatedCredentials','no')) {
+    push @cfgarr, 'GSSAPIStoreDelegatedCredentials no';
+}
+if(sshd_supports_opt('GSSCleanupCreds','yes')) {
+    push @cfgarr, 'GSSCleanupCreds yes';
+}
+if(sshd_supports_opt('GSSUseSessionCredCache','no')) {
+    push @cfgarr, 'GSSUseSessionCredCache no';
+}
 push @cfgarr, '#';
 
 
@@ -527,14 +567,20 @@ if(sshd_supports_opt('AcceptEnv','')) {
 }
 if(sshd_supports_opt('AddressFamily','any')) {
     # Address family must be specified before ListenAddress
-    splice @cfgarr, 13, 0, 'AddressFamily any';
+    splice @cfgarr, 14, 0, 'AddressFamily any';
 }
 if(sshd_supports_opt('Compression','no')) {
     push @cfgarr, 'Compression no';
 }
+if(sshd_supports_opt('KbdInteractiveAuthentication','no')) {
+    push @cfgarr, 'KbdInteractiveAuthentication no';
+}
 if(sshd_supports_opt('KeepAlive','no')) {
     push @cfgarr, 'KeepAlive no';
 }
+if(sshd_supports_opt('LookupClientHostnames','no')) {
+    push @cfgarr, 'LookupClientHostnames no';
+}
 if(sshd_supports_opt('MaxAuthTries','10')) {
     push @cfgarr, 'MaxAuthTries 10';
 }
@@ -547,6 +593,9 @@ if(sshd_supports_opt('PermitTunnel','no')) {
 if(sshd_supports_opt('PermitUserEnvironment','no')) {
     push @cfgarr, 'PermitUserEnvironment no';
 }
+if(sshd_supports_opt('RhostsAuthentication','no')) {
+    push @cfgarr, 'RhostsAuthentication no';
+}
 if(sshd_supports_opt('TCPKeepAlive','no')) {
     push @cfgarr, 'TCPKeepAlive no';
 }
@@ -556,8 +605,16 @@ if(sshd_supports_opt('UseDNS','no')) {
 if(sshd_supports_opt('UsePAM','no')) {
     push @cfgarr, 'UsePAM no';
 }
-if(sshd_supports_opt('UsePrivilegeSeparation','no')) {
-    push @cfgarr, 'UsePrivilegeSeparation no';
+
+if($sshdid =~ /OpenSSH/) {
+    # http://bugs.opensolaris.org/bugdatabase/view_bug.do?bug_id=6492415
+    if(sshd_supports_opt('UsePrivilegeSeparation','no')) {
+        push @cfgarr, 'UsePrivilegeSeparation no';
+    }
+}
+
+if(sshd_supports_opt('VerifyReverseMapping','no')) {
+    push @cfgarr, 'VerifyReverseMapping no';
 }
 if(sshd_supports_opt('X11UseLocalhost','yes')) {
     push @cfgarr, 'X11UseLocalhost yes';
@@ -635,6 +692,7 @@ if(! -e $knownhosts) {
 #  ConnectTimeout                    : OpenSSH 3.7.0 and later
 #  ControlMaster                     : OpenSSH 3.9.0 and later
 #  ControlPath                       : OpenSSH 3.9.0 and later
+#  DisableBanner                     :  SunSSH 1.2.0 and later
 #  DynamicForward                    : OpenSSH 2.9.0 and later
 #  EnableSSHKeysign                  : OpenSSH 3.6.0 and later
 #  EscapeChar                        : OpenSSH 1.2.1 and later [3]
@@ -654,6 +712,7 @@ if(! -e $knownhosts) {
 #  HostName                          : OpenSSH 1.2.1 and later
 #  IdentitiesOnly                    : OpenSSH 3.9.0 and later
 #  IdentityFile                      : OpenSSH 1.2.1 and later
+#  IgnoreIfUnknown                   :  SunSSH 1.2.0 and later
 #  KeepAlive                         : OpenSSH 1.2.1 and later
 #  KbdInteractiveAuthentication      : OpenSSH 2.3.0 and later
 #  KbdInteractiveDevices             : OpenSSH 2.3.0 and later [3]
@@ -760,6 +819,10 @@ if(($sshid =~ /OpenSSH/) && ($sshvernum >= 420)) {
     push @cfgarr, 'ControlPath none';
 }
 
+if(($sshid =~ /SunSSH/) && ($sshvernum >= 120)) {
+    push @cfgarr, 'DisableBanner yes';
+}
+
 if(($sshid =~ /OpenSSH/) && ($sshvernum >= 360)) {
     push @cfgarr, 'EnableSSHKeysign no';
 }
@@ -791,6 +854,10 @@ if(($sshid =~ /OpenSSH/) && ($sshvernum >= 390)) {
     push @cfgarr, 'IdentitiesOnly yes';
 }
 
+if(($sshid =~ /SunSSH/) && ($sshvernum >= 120)) {
+    push @cfgarr, 'IgnoreIfUnknown no';
+}
+
 if((($sshid =~ /OpenSSH/) && ($sshvernum < 380)) ||
     ($sshid =~ /SunSSH/)) {
     push @cfgarr, 'KeepAlive no';
@@ -843,12 +910,55 @@ if($error) {
     logmsg $error;
     exit 1;
 }
+
+
+#***************************************************************************
+# Initialize client sftp config with options actually supported.
+#
+logmsg 'generating sftp client config file...' if($verbose);
+splice @cfgarr, 1, 1, "# $sshverstr sftp client configuration file for curl testing";
+#
+for(my $i = scalar(@cfgarr) - 1; $i > 0; $i--) {
+    if($cfgarr[$i] =~ /^DynamicForward/) {
+        splice @cfgarr, $i, 1;
+        next;
+    }
+    if($cfgarr[$i] =~ /^ClearAllForwardings/) {
+        splice @cfgarr, $i, 1, "ClearAllForwardings yes";
+        next;
+    }
+}
+
+
+#***************************************************************************
+# Write out resulting sftp client configuration file for curl's tests
+#
+$error = dump_array($sftpconfig, @cfgarr);
+if($error) {
+    logmsg $error;
+    exit 1;
+}
+@cfgarr = ();
+
+
+#***************************************************************************
+# Generate client sftp commands batch file for sftp server verification
+#
+logmsg 'generating sftp client commands file...' if($verbose);
+push @cfgarr, 'pwd';
+push @cfgarr, 'quit';
+$error = dump_array($sftpcmds, @cfgarr);
+if($error) {
+    logmsg $error;
+    exit 1;
+}
 @cfgarr = ();
 
 
 #***************************************************************************
 # Start the ssh server daemon without forking it
 #
+logmsg "SCP/SFTP server listening on port $port" if($verbose);
 my $rc = system "$sshd -e -D -f $sshdconfig > $sshdlog 2>&1";
 if($rc == -1) {
     logmsg "$sshd failed with: $!";
@@ -866,7 +976,7 @@ elsif($verbose && ($rc >> 8)) {
 # Clean up once the server has stopped
 #
 unlink($hstprvkeyf, $hstpubkeyf, $cliprvkeyf, $clipubkeyf, $knownhosts);
-unlink($sshdconfig, $sshconfig);
+unlink($sshdconfig, $sshconfig, $sftpconfig);
 
 
 exit 0;
